<?php
namespace Bling\App\Model;

use Bling\App\Config\Autenticacao;
use Bling\App\Config\Http\Api;
use Bling\App\Config\Http\Request;
use Bling\App\Config\Http\Response\RespostaCriacao;

class ModuloCampo {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}

class TipoCampoCustomizado {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}
class TamanhoCampoCustomizado {
    public $minimo;
    public $maximo;

    public function __construct(int $minimo, int $maximo) {
        $this->minimo = $minimo;
        $this->maximo = $maximo;
    }
}
class OpcaoCampoCustomizado {
    public $id;
    public $nome;

    public function __construct(int $id, int $nome) {
        $this->id = $id;
        $this->nome = $nome;
    }
}
class AgrupadorCampoCustomizado {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}
class CampoCustomizadoModulo {
    public $id;
    public $nome;
    public $situacao;
}

class CampoCustomizado {
    private $id;
    public $nome;
    public $modulo;
    public $tipoCampo;
    public $placeholder;
    public $situacao;
    public $obrigatorio;
    public $tamanho;
    public $agrupadores;
    public $opcoes;

    /**
     * @param ?AgrupadorCampoCustomizado[] $agrupadores
     * @param ?OpcaoCampoCustomizado[] $opcoes
     */
    public function __construct(string $nome, ?ModuloCampo $modulo, ?TipoCampoCustomizado $tipoCampo, ?string $placeholder = null,
        ?string $situacao = null, ?bool $obrigatorio = null, ?TamanhoCampoCustomizado $tamanho = null, ?array $agrupadores = null, ?array $opcoes = null)
    {
        $this->nome = $nome;
        $this->modulo = $modulo;
        $this->tipoCampo = $tipoCampo;
        $this->placeholder = $placeholder;
        $this->situacao = $situacao;
        $this->obrigatorio = $obrigatorio;
        $this->tamanho = $tamanho;
        $this->agrupadores = $agrupadores;
        $this->opcoes = $opcoes;
    }

    /**
     * @return RespostaCriacao|ApiError
     */
    public function cadastrar() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/campos-customizados", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $this->toString()
        );
        return $response->http_code == 201 ? $response->body->data : $response->body;
    }

    /**
     * @return CampoCustomizado|ApiError
     */
    public static function buscar(int $idCampo) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/campos-customizados/$idCampo", [
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    public function setId(int $id) {
        $this->id = $id;
    }
    
    public function getId() {
        return $this->id;
    }

    /**
     * @return true|ApiError
     */
    public static function atualizar(CampoCustomizado $campoCustomizado) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::put("$url/campos-customizados/{$campoCustomizado->getId()}", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $campoCustomizado->toString()
        );
        return $response->http_code == 200 ? true : $response->body;
    }

    /**
     * @return ModuloCampoCustomizado[]|ApiError
     */
    public static function modulos() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/campos-customizados/modulos", [
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return CampoCustomizadoModulo[]
     */
    public static function camposModulos(int $idModulo) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/campos-customizados/modulos/$idModulo", [
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    public function toString() {
        $categoria = [];
        foreach($this as $chave => $valor) {
            if($chave != "id" && $valor !== null) {
                $categoria[$chave] = $valor;
            }
        }
        return json_encode($categoria);
    }
}