<?php
namespace Bling\App\Model;
use Bling\App\Config\Autenticacao;
use Bling\App\Config\Http\Request;
use Bling\App\Config\Http\Api;
use Bling\App\Config\Http\ApiError;
use Bling\App\Config\Http\Response\RespostaCriacao;

class CategoriaPaiProduto {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}

/**
 * @var int $id
 * @var string $descrição
 * @var CategoriaPaiProduto $categoriaPai
 */
class CategoriaProduto {
    private $id;
    public $descricao;
    public $categoriaPai;

    public function __construct(string $descricao, ?CategoriaPaiProduto $categoriaPai = null) {
        $this->descricao = $descricao;
        $this->categoriaPai = $categoriaPai;
    }

    /**
     * @return RespostaCriacao|ApiError
     */
    public function cadastrar() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/categorias/produtos", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $this->toString()
        );
        return $response->http_code == 201 ? $response->body->data : $response->body;
    }

    public function setId(int $id) {
        $this->id = $id;
    }
    
    public function getId() {
        return $this->id;
    }

    /**
     * @return CategoriaProduto
     */
    public static function buscar(int $id) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/categorias/produtos/$id", [
            "Content-Type: application/json",
            "Authorization: Bearer $tokenAcesso",
            "Accept: application/json"
        ]);
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return CategoriaProduto[]|ApiError
     */
    public static function buscarTodas(int $pagina, $limite = 100) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/categorias/produtos?pagina=$pagina&limite=$limite", [
            "Content-Type: application/json",
            "Authorization: Bearer $tokenAcesso",
            "Accept: application/json"
        ]);
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return true|ApiError
     */
    public static function atualizar(CategoriaProduto $categoria) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $idCategoria = $categoria->getId();
        $response = Request::put("$url/categorias/produtos/$idCategoria", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $categoria->toString()
        );
        return $response->http_code == 204 ? true : $response->body;
    }

    /**
     * @return true|ApiError
     */
    public static function excluir(int $idCategoria) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::delete("$url/categorias/produtos/$idCategoria", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 204 ? true : $response->body;
    }

    public function toString() {
        $categoria = [];
        foreach($this as $chave => $valor) {
            if($valor !== null) {
                $categoria[$chave] = $valor;
            }
        }
        return json_encode($categoria);
    }
}
?>