<?php
namespace Bling\App\Model\Type\Product;

use Bling\App\Config\Autenticacao;
use Bling\App\Config\Http\Api;
use Bling\App\Config\Http\Request;

class CategoriaLoja {
    public $id;

    public function __construct($id) {
        $this->id = $id;
    }
}

class idCategoriaProduto {
    public $id;

    public function __construct($id) {
        $this->id = $id;
    }
}

class FiltroCategoriaProdutoLoja {
    public $pagina;
    public $limite;
    public $idLoja;
    public $idCategoriaProduto;
    public $idCategoriaProdutoPai;

    public function __construct(int $pagina, int $limite = 100, ?int $idLoja = null, ?int $idCategoriaProduto = null, ?int $idCategoriaProdutoPai = null) {
        $this->pagina = $pagina;
        $this->limite = $limite;
        $this->idLoja = $idLoja;
        $this->idCategoriaProduto = $idCategoriaProduto;
        $this->idCategoriaProdutoPai = $idCategoriaProdutoPai;
    }

    public function toUrlEncode() {
        $qtdParametrosInseridos = 0;
        $queryParams = "";

        foreach($this as $chave => $valor) {
            if($valor !== null) {
                $queryParams .= $qtdParametrosInseridos == 0 ? "?$chave=$valor" : "&$chave=$valor";
                $qtdParametrosInseridos++;
            }
        }
        return $queryParams;
    }
}


class CategoriaProdutoLoja {
    public $id;
    public $loja;
    public $descricao;
    public $codigo;
    public $categoriaProduto;

    public function __construct(CategoriaLoja $loja, string $descricao, string $codigo, idCategoriaProduto $categoriaProduto) {
        $this->loja = $loja;
        $this->descricao = $descricao;
        $this->codigo = $codigo;
        $this->categoriaProduto = $categoriaProduto;
    }

    /**
     * @return RespostaCriacao|ApiError
     */
    public function cadastrar() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/categorias/lojas", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $this->toString()
        );
        return $response->http_code == 201 ? $response->body->data : $response->body;
    }

    public function setId(int $id) {
        $this->id = $id;
    }
    
    public function getId() {
        return $this->id;
    }

    /**
     * @return CategoriaProdutoLoja|ApiError
     */
    public static function buscar(int $idCategoriaProdutoLoja) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/categorias/lojas/$idCategoriaProdutoLoja", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return CategoriaProdutoLoja[]|ApiError
     */
    public static function buscarTodas(FiltroCategoriaProdutoLoja $filtro) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/categorias/lojas", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return true|ApiError
     */
    public static function atualizar(CategoriaProdutoLoja $categoriaProdutoLoja) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::put("$url/categorias/lojas/{$categoriaProdutoLoja->getId()}", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $categoriaProdutoLoja->toString()
        );
        return $response->http_code == 204 ? true : $response->body;
    }

    /**
     * @return true|ApiError
     */
    public static function excluir(int $idCategoriaProdutoLoja) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::delete("$url/categorias/lojas/$idCategoriaProdutoLoja", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 204 ? true : $response->body;
    }

    public function toString() {
        $categoria = [];
        foreach($this as $chave => $valor) {
            if($chave != "id" && $valor !== null) {
                $categoria[$chave] = $valor;
            }
        }
        return json_encode($categoria);
    }
}
?>