<?php
namespace Bling\App\Model;

use Bling\App\Model\Endereco;
use Bling\App\Model\Type\Contact\DadosAdicionais;
use Bling\App\Model\Type\Contact\VendedorContato;
use Bling\App\Model\Type\Contact\FinanceiroContato;
use Bling\App\Model\Type\Contact\PaisContato;
use Bling\App\Model\Type\Contact\TipoContato;
use Bling\App\Model\Type\Contact\FiltroContato;
use Bling\App\Model\Type\Contact\AtualizarSituacoesContatos;
use Bling\App\Config\Http\Request;
use Bling\App\Config\Http\Response\RespostaCriacao;
use Bling\App\Config\Autenticacao;
use Bling\App\Config\Http\Api;
use Bling\App\Config\Http\ApiError;

class Contato {
    private $id;
    public $nome;
    public $codigo;
    public $situacao;
    public $numeroDocumento;
    public $telefone;
    public $celular;
    public $fantasia;
    public $tipo;
    public $indicadorIe;
    public $ie;
    public $rg;
    public $orgaoEmissor;
    public $email;
    public $endereco;
    public $vendedor;
    public $dadosAdicionais;
    public $financeiro;
    public $pais;
    public $tiposContato;
    public $pessoasContato;

    /**
     * @param string $situacao A - Ativo | I - Inativo | E - Excluído | S - Sem movimentação
     * @param string $tipo J - Jurídica | F - Física | E - Estrangeira
     * @param ?string $indicadorIe 1- Contribuinte ICMS | 2- Contribuinte isento | 9- Não Contribuinte
     * @param ?TipoContato[] $tiposContato
     * @param ?TipoContato[] $pessoasContato
     */
    public function __construct(string $nome, string $codigo, string $situacao, string $numeroDocumento, string $tipo, string $celular,
        string $telefone, string $email, ?array $tiposContato = null, ?string $indicadorIe = null, ?Endereco $endereco = null,
        ?string $fantasia = null, ?string $ie = null, ?string $rg = null, ?string $orgaoEmissor = null, ?VendedorContato $vendedor = null,
        ?FinanceiroContato $financeiro = null, ?DadosAdicionais $dadosAdicionais = null, ?PaisContato $pais = null, ?array $pessoasContato = null)
    {
        $this->nome = $nome;
        $this->codigo = $codigo;
        $this->situacao = $situacao;
        $this->numeroDocumento = $numeroDocumento;
        $this->telefone = $telefone;
        $this->celular = $celular;
        $this->fantasia = $fantasia;
        $this->tipo = $tipo;
        $this->indicadorIe = $indicadorIe;
        $this->ie = $ie;
        $this->rg = $rg;
        $this->orgaoEmissor = $orgaoEmissor;
        $this->email = $email;
        $this->endereco = $endereco;
        $this->vendedor = $vendedor;
        $this->dadosAdicionais = $dadosAdicionais;
        $this->financeiro = $financeiro;
        $this->pais = $pais;
        $this->tiposContato = $tiposContato;
        $this->pessoasContato = $pessoasContato;
    }

    /**
     * @return RespostaCriacao|ApiError
     */
    public function cadastrar() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/contatos", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $this->toString()
        );
        return $response->http_code == 201 ? $response->body->data : $response->body;
    }

    public function setId(int $id) {
        $this->id = $id;
    }

    public function getId() {
        return $this->id;
    }

    /**
     * @return Contato|ApiError
     */
    public static function buscar(string $idContato) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/contatos/$idContato", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ]
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return Contato[]|ApiError
     */
    public static function filtrar(FiltroContato $filtroContato) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/contatos".$filtroContato->toUrlEncode(), [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ]
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return TipoContato[]|ApiError
     */
    public static function tipos(int $idContato) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/contatos/$idContato/tipos", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ]
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    public static function atualizar(Contato $contato) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $idContato = $contato->getId();
        $response = Request::put("$url/contatos/$idContato", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $contato->toString()
        );
        return $response->http_code == 204;
    }

    public static function atualizarSituacoes(AtualizarSituacoesContatos $idsContatos) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/contatos/situacoes", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            json_encode($idsContatos)
        );
        return $response->http_code == 204;
    }

    /**
     * @param int[] $idsContatos
     */
    public static function excluir(array $idsContatos) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $ids = implode(",", $idsContatos);
        $response = Request::delete("$url/contatos?idsContatos=$ids", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ]
        );
        return $response->http_code == 204;
    } 

    public function toString() {
        $categoria = [];
        foreach($this as $chave => $valor) {
            if($chave != "id" && $valor !== null) {
                $categoria[$chave] = $valor;
            }
        }
        return json_encode($categoria);
    }
}
?>