<?php
namespace Bling\App\Model;

use Bling\App\Config\Autenticacao;
use Bling\App\Config\Http\Api;
use Bling\App\Config\Http\ApiError;
use Bling\App\Config\Http\Request;
use Bling\App\Config\Http\Response\RespostaCriacao;
use Bling\App\Config\Http\Response\RespostaDeposito;
use Bling\App\Model\Type\Product\FiltroDeposito;

class Deposito {
    private $id;
    public $descricao;
    public $situacao;
    public $padrao;
    public $desconsiderarSaldo;

    /**
     * @param int $situacao 0- Inativo | 1- Ativo
     */
    public function __construct(string $descricao, int $situacao, bool $padrao, bool $desconsiderarSaldo){
        $this->descricao = $descricao;
        $this->situacao = $situacao;
        $this->padrao = $padrao;
        $this->desconsiderarSaldo = $desconsiderarSaldo;
    }

    /**
     * @return RespostaCriacao|ApiError
     */
    public function cadastrar() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/depositos", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $this->toString()
        );
        return $response->http_code == 201 ? $response->body->data : $response->body;
    }

    public function setId(int $id) {
        $this->id = $id;
    }
    
    public function getId() {
        return $this->id;
    }

    /**
     * @return RespostaDeposito|ApiError
     */
    public static function atualizar(Deposito $deposito) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::put("$url/depositos/{$deposito->getId()}", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $deposito->toString()
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }
    
    /**
     * @return Deposito[]|ApiError
     */
    public static function buscar(int $idDeposito) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/depositos/$idDeposito", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return Deposito[]|ApiError
     */
    public static function filtrar(FiltroDeposito $filtro) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/depositos{$filtro->toUrlEncode()}", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    public function toString() {
        $categoria = [];
        foreach($this as $chave => $valor) {
            if($chave != "id" && $valor !== null) {
                $categoria[$chave] = $valor;
            }
        }
        return json_encode($categoria);
    }
}
?>