<?php
namespace Bling\App\Model\Type\Product;

use Bling\App\Config\Autenticacao;
use Bling\App\Config\Http\Request;
use Bling\App\Config\Http\Api;
use Bling\App\Config\Http\ApiError;
use Bling\App\Config\Http\Response\RespostaCriacao;
use Bling\App\Model\Type\Product\FiltroFornecedorProduto;

class ProdutoFornecedorDTO {
    public $id;

    function __construct(int $id) {
        $this->id = $id;
    }
}

class FornecedorDTO {
    public $id;

    function __construct(int $id) {
        $this->id = $id;
    }
}

class FornecedorProduto {
    public $id;
    public $descricao;
    public $codigo;
    public $precoCusto;
    public $precoCompra;
    public $padrao;
    public $produto;
    public $fornecedor;
    public $garantia;

    function __construct(ProdutoFornecedorDTO $produto, FornecedorDTO $fornecedor, ?bool $padrao = null, ?float $precoCompra = null,
        ?float $precoCusto = null, ?string $descricao = null, ?string $codigo = null, ?int $garantia = null
    ){
        $this->descricao = $descricao;
        $this->codigo = $codigo;
        $this->precoCusto = $precoCusto;
        $this->precoCompra = $precoCompra;
        $this->padrao = $padrao;
        $this->produto = $produto;
        $this->fornecedor = $fornecedor;
        $this->garantia = $garantia; 
    }

    /**
     * @return RespostaCriacao|ApiError
     */
    public function cadastrar() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/produtos/fornecedores", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $this->toString()
        );
        return $response->http_code == 201 ? $response->body->data : $response->body;
    }

    public function setId(int $id) {
        $this->id = $id;
    }
    
    public function getId() {
        return $this->id;
    }

    /**
     * @return FornecedorProduto|ApiError
     */
    public static function buscar(int $idFornecedorProduto) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/produtos/fornecedores/$idFornecedorProduto", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return FornecedorProduto[]|ApiError
     */
    public static function filtrar(FiltroFornecedorProduto $filtro) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/produtos/fornecedores".$filtro->toUrlEncode(), [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return RespostaCriacao|ApiError
     */
    public static function atualizar(FornecedorProduto $fornecedorProduto) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $idFornecedorProduto = $fornecedorProduto->getId();
        $response = Request::put("$url/produtos/fornecedores/$idFornecedorProduto", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $fornecedorProduto->toString()
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    /**
     * @return true|ApiError
     */
    public static function excluir(int $idFornecedorProduto) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::delete("$url/produtos/fornecedores/$idFornecedorProduto", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 204 ? true : $response->body;
    }

    public function toString() {
        $categoria = [];
        foreach($this as $chave => $valor) {
            if($chave != "id" && $valor !== null) {
                $categoria[$chave] = $valor;
            }
        }
        return json_encode($categoria);
    }
}
?>