<?php
namespace Bling\App\Model;

use Bling\App\Config\Autenticacao;
use Bling\App\Config\Http\Request;
use Bling\App\Config\Http\Api;
use Bling\App\Config\Http\ApiError;
use Bling\App\Config\Http\Response\RespostaProduto;
use Bling\App\Model\Type\Product\EstruturaProduto;
use Bling\App\Model\CategoriaPaiProduto;
use Bling\App\Model\Type\Product\ProdutoEstoque;
use Bling\App\Model\Type\Product\Dimensao;
use Bling\App\Model\Tributacao;
use Bling\App\Model\Type\Product\CampoCustomizadoProduto;
use Bling\App\Model\Type\Product\FiltroProduto;
use Bling\App\Model\Type\Product\MidiaProduto;
use Bling\App\Model\Type\Product\LinhaProduto;
use Bling\App\Model\ProdutoFilho;

class Produto {
    private $id;
    public $nome;
    public $tipo;
    public $situacao;
    public $formato;
    public $descricao;
    public $descricaoCurta;
    public $midia;
    public $pesoBruto;
    public $dimensoes;
    public $marca;
    public $gtin;
    public $idFabricante;
    public $descricaoFornecedor;
    public $codigo;
    public $preco;
    public $categoria;
    public $estoque;
    public $unidade;
    public $freteGratis;
    public $pesoLiquido;
    public $condicao;
    public $observacoes;
    public $volumes;
    public $itensPorCaixa;
    public $dataValidade;
    public $descricaoComplementar;
    public $gtinEmbalagem;
    public $tipoProducao;
    public $tributacao;
    public $variacoes;
    public $actionEstoque;
    public $linkExterno;
    public $linhaProduto;
    public $estrutura;
    public $camposCustomizados;

    /**
     * @param string $tipo S - Servio | P - Produto | N - Servio de nota 06, 21, 22
     * @param string $situacao A - Ativo | I - Inativo
     * @param string $formato S - Simple | V - Com variaes | E - Com composio
     * @param ?string $condicao 0 - No especificado | 1 - Novo | 2- Usado
     * @param ?string $tipoProducao T - Terceiro | P - Prpria
     * @param ?ProdutoFilho[] $variacoes
     * @param ?string $actionEstoque Z - Zerar estoque | T - Transfere o estoque do produto pai para a primeira variao informada
     * @param ?CampoCustomizadoProduto[] $camposCustomizados
     */
    public function __construct(string $nome, string $tipo, string $situacao, string $formato, string $descricao, MidiaProduto $midia,
        float $pesoBruto, Dimensao $dimensoes, string $marca, string $gtin, string $descricaoFornecedor, ?string $codigo = null, //int $idFabricante,
        ?float $preco = null, ?CategoriaPaiProduto $categoria = null, ?ProdutoEstoque $estoque = null, ?string $unidade = null, ?bool $freteGratis = null,
        ?float $pesoLiquido = null, ?string $condicao = null, ?string $observacoes = null, ?float $volumes = null, ?float $itensPorCaixa = null,
        ?string $dataValidade = null, ?string $descricaoComplementar = null, ?string $gtinEmbalagem = null, ?string $tipoProducao = null,
        ?Tributacao $tributacao = null, ?array $variacoes = null, ?string $actionEstoque = null, ?string $linkExterno = null,
        ?LinhaProduto $linhaProduto = null, ?EstruturaProduto $estrutura = null, ?array $camposCustomizados = null)
    {
        $this->nome = $nome;
        $this->tipo = $tipo;
        $this->situacao = $situacao;
        $this->formato = $formato;
        $this->descricao = $descricao;
        $this->descricaoCurta = $descricao;
        $this->midia = $midia;
        $this->pesoBruto = $pesoBruto;
        $this->dimensoes = $dimensoes;
        $this->marca = $marca;
        $this->gtin = $gtin;
        // $this->idFabricante = $idFabricante;
        $this->descricaoFornecedor = $descricaoFornecedor;
        $this->codigo = $codigo;
        $this->preco = $preco;
        $this->categoria = $categoria;
        $this->estoque = $estoque;
        $this->unidade = $unidade;
        $this->freteGratis = $freteGratis;
        $this->pesoLiquido = $pesoLiquido;
        $this->condicao = $condicao;
        $this->observacoes = $observacoes;
        $this->volumes = $volumes;
        $this->itensPorCaixa = $itensPorCaixa;
        $this->dataValidade = $dataValidade;
        $this->descricaoComplementar = $descricaoComplementar;
        $this->gtinEmbalagem = $gtinEmbalagem;
        $this->tipoProducao = $tipoProducao;
        $this->tributacao = $tributacao;
        $this->variacoes = $variacoes;
        $this->actionEstoque = $actionEstoque;
        $this->linkExterno = $linkExterno;
        $this->linhaProduto = $linhaProduto;
        $this->estrutura = $estrutura;
        $this->camposCustomizados = $camposCustomizados;
    }

    /**
     * @return RespostaProduto|ApiError
     */
    public function cadastrar() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/produtos", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $this->toString()
        );
        return $response->http_code == 201 ? $response->body->data : $response->body;
    }
    
    public function setId(int $id) {
        $this->id = $id;
    }
    
    public function getId() {
        return $this->id;
    }
    
    /**
     * @return true|ApiError
     */
    public static function atualizar(Produto $produto) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::put("$url/produtos/{$produto->getId()}", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $produto->toString()
        );
        return $response->http_code == 200 ? true : $response->body;
    }

    /**
     * @return Produto|ApiError
     */
    public static function buscar(int $idProduto) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/produtos/$idProduto", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }
    
    /**
     * @return Produto[]|ApiError
     */
    public static function filtrar(FiltroProduto $filtro) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::get("$url/produtos{$filtro->toUrlEncode()}", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    public function toString() {
        $categoria = [];
        foreach($this as $chave => $valor) {
            if($chave != "id" && $valor !== null) {
                $categoria[$chave] = $valor;
            }
        }
        return json_encode($categoria);
    }
}