<?php
namespace Bling\App\Model\Type\Product;

use Bling\App\Config\Autenticacao;
use Bling\App\Config\Http\Api;
use Bling\App\Config\Http\Request;
use Bling\App\Config\Http\Response\RespostaCriacao;
use Bling\App\Config\Http\Response\RespostaProdutoLoja;

class ProdutoLojaDTO {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}
class LojaProdutoLojaDTO {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}
class CategoriaProdutoLojaDTO {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}
class FornecedorProdutoLojaDTO {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}
class MarcaLojaProdutoLojaDTO {
    public $id;

    public function __construct(int $id) {
        $this->id = $id;
    }
}


class ProdutoLoja {
    public $id;
    public $codigo;
    public $preco;
    public $precoPromocional;
    public $produto;
    public $loja;
    public $fornecedorLoja;
    public $marcaLoja;
    public $categoriasProdutos;

    /**
     * @param CategoriaProdutoLojaDTO[] $categoriasProdutos
     */
    public function __construct(string $codigo, ProdutoLojaDTO $produto, LojaProdutoLojaDTO $loja, ?float $preco = null, ?float $precoPromocional = null,
        array $categoriasProdutos = null, ?FornecedorProdutoLojaDTO $fornecedorLoja = null, ?MarcaLojaProdutoLojaDTO $marcaLoja = null
    ) {
        $this->codigo = $codigo;
        $this->preco = $preco;
        $this->precoPromocional = $precoPromocional;
        $this->produto = $produto;
        $this->loja = $loja;
        $this->fornecedorLoja = $fornecedorLoja;
        $this->marcaLoja = $marcaLoja;
        $this->categoriasProdutos = $categoriasProdutos;
    }

    /**
     * @return RespostaCriacao|ApiError
     */
    public function cadastrar() {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::post("$url/produtos/lojas", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $this->toString()
        );
        return $response->http_code == 201 ? $response->body->data : $response->body;
    }
    /**
     * @return RespostaProdutoLoja|ApiError
     */
    public static function atualizar(ProdutoLoja $produtoLoja) {
        $url = Api::getUrl();
        $autenticacao = new Autenticacao();
        $tokenAcesso = $autenticacao->getAccessToken();
        $response = Request::put("$url/produtos/lojas/{$produtoLoja->getId()}", [
                "Content-Type: application/json",
                "Authorization: Bearer $tokenAcesso",
                "Accept: application/json"
            ],
            $produtoLoja->toString()
        );
        return $response->http_code == 200 ? $response->body->data : $response->body;
    }

    public function setId(int $id) {
        $this->id = $id;
    }
    
    public function getId() {
        return $this->id;
    }

    public function toString() {
        $categoria = [];
        foreach($this as $chave => $valor) {
            if($chave != "id" && $valor !== null) {
                $categoria[$chave] = $valor;
            }
        }
        return json_encode($categoria);
    }
}

?>