<?php
namespace Ifood\App\Config;
use Ifood\App\Config\Http\Request;
use Ifood\App\Config\Http\Api;
use Ifood\App\Config\Autenticacao;

class Loja {
    public static function detalhes(string $idLojaIfood) {
        $url = Api::getUrl()."/merchant/".Api::getVersao()."/merchants/$idLojaIfood";
        $autenticacao = new Autenticacao();
        $buscaDetalhesLoja = Request::get($url, $autenticacao->token);
        
        return $buscaDetalhesLoja->http_code == 200 ? $buscaDetalhesLoja : null;
    }

    public static function status(string $idLojaIfood) {
        $url = Api::getUrl()."/merchant/".Api::getVersao()."/merchants/$idLojaIfood/status";
        $autenticacao = new Autenticacao();

        $merchant = Request::get($url, $autenticacao->token);
        return $merchant->body;
    }

    public static function horarios(string $idLoja) {
        $url = Api::getUrl()."/merchant/".Api::getVersao()."/merchants/$idLoja/opening-hours";
        $autenticacao = new Autenticacao();

        $horarios = Request::get($url, $autenticacao->token);
        return $horarios->http_code == 200 ? $horarios->body : [];
    }

    public static function atualizarHorarios(string $idLoja, array $horarios) {
        $url = Api::getUrl()."/merchant/".Api::getVersao()."/merchants/$idLoja/opening-hours";
        $autenticacao = new Autenticacao();
        $corpoRequisicao = json_encode([
            "storeId"=> $idLoja,
            "shifts"=> $horarios
        ]);

        $atualizarHorarios = Request::put($url, $autenticacao->token, $corpoRequisicao, "json");
        return $atualizarHorarios->http_code == 201 ? true : $atualizarHorarios->body;
    }

    public static function criarPausa(string $idLoja, string $titulo, string $dataInicio, string $dataFim) {
        $url = Api::getUrl()."/merchant/".Api::getVersao()."/merchants/$idLoja/interruptions";
        $autenticacao = new Autenticacao();

        $dados = json_encode([
            "description"=> $titulo,
            "start"=> $dataInicio,
            "end"=> $dataFim
        ]);
        $criarPausa = Request::post($url, $autenticacao->token, $dados, "json");
        return $criarPausa->body;
    }

    public static function pausas(string $idLoja) {
        $url = Api::getUrl()."/merchant/".Api::getVersao()."/merchants/$idLoja/interruptions";
        $autenticacao = new Autenticacao();

        $pausas = Request::get($url, $autenticacao->token);
        return $pausas->body;
    }

    public static function excluirPausa(string $idLoja, string $idPausa) {
        $url = Api::getUrl()."/merchant/".Api::getVersao()."/merchants/$idLoja/interruptions/$idPausa";
        $autenticacao = new Autenticacao();

        $excluirPausa = Request::delete($url, $autenticacao->token);
        return $excluirPausa;
    }
}