<?php
namespace Ifood\App\Config\Http;

class Request {
    private static $curl;

    function __construct(){}

    /**
     * Requisio post
     *
     * @param string $url Endpoint de destino
     * @param string $token Token de autenticao
     * @param string $dados Corpo da requisio
     * @param string $tipoConteudo Tipo do conteudo (x-www-form-urlencoded ou json)
     * @return object Cdigo http e body da resposta
     */
    public static function post(string $url, ?string $token, ?string $dados, string $tipoConteudo = "") {
        $tiposConteudo = ["x-www-form-urlencoded", "json"];
        $cabecalhoSolicitacao = [
            "Authorization: Bearer $token",
        ];
        if(in_array($tipoConteudo, $tiposConteudo)) {
            array_push($cabecalhoSolicitacao, "Content-Type: application/$tipoConteudo");
        }
        if(!$dados) {
            array_push($cabecalhoSolicitacao, "Content-Length: 0");
        }
        $configuracoesRequisicao = [
            CURLOPT_URL => "$url",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_HTTPHEADER => $cabecalhoSolicitacao,
            // CURLOPT_POSTFIELDS => $dados
        ];
        if(!is_null($dados)) {
            $configuracoesRequisicao[CURLOPT_POSTFIELDS] = $dados;
        }

        self::$curl = curl_init();
        curl_setopt_array(self::$curl, $configuracoesRequisicao);
          
        $response = curl_exec(self::$curl);
        $httpcode = curl_getinfo(self::$curl)['http_code'];
        curl_close(self::$curl);

        if(substr($httpcode, 0, 1) == 5) {
            Request::salvarErro($url, "POST", "$httpcode", $response);
        }

        return (object)["http_code"=> $httpcode, "body"=> json_decode($response)];
    }

    /**
     * Requisio put
     *
     * @param string $url Endpoint de destino
     * @param string $token Token de autenticao
     * @param string $dados Corpo da requisio
     * @param string $tipoConteudo Tipo do conteudo (x-www-form-urlencoded ou json)
     * @return object Cdigo http e body da resposta
     */
    public static function put(string $url, string $token, string $dados, string $tipoConteudo = "json") {
        $tiposConteudo = ["x-www-form-urlencoded", "json"];
        $cabecalhoSolicitacao = [
            "Authorization: Bearer $token",
        ];
        if(in_array($tipoConteudo, $tiposConteudo)) {
            array_push($cabecalhoSolicitacao, "Content-Type: application/$tipoConteudo");
        }
        $configuracoesRequisicao = [
            CURLOPT_URL => "$url",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'PUT',
            CURLOPT_HTTPHEADER => $cabecalhoSolicitacao,
            CURLOPT_POSTFIELDS => $dados
        ];

        self::$curl = curl_init();
        curl_setopt_array(self::$curl, $configuracoesRequisicao);
          
        $response = curl_exec(self::$curl);
        $httpcode = curl_getinfo(self::$curl)['http_code'];
        curl_close(self::$curl);

        if(substr($httpcode, 0, 1) == 5) {
            Request::salvarErro($url, "PUT", "$httpcode", $response);
        }

        return (object)["http_code"=> $httpcode, "body"=> json_decode($response)];
    }

    /**
     * Requisio get
     *
     * @param string $url Endpoint de destino
     * @param string $token Token de autenticao
     * @return object Cdigo http e body da resposta
     */
    public static function get($url, $token, $encode = true, ?string $merchantId = null) {
        self::$curl = curl_init();
        $cabecalhoSolicitacao = [
            "Authorization: Bearer $token"
        ];
        if($merchantId != null) {
            array_push($cabecalhoSolicitacao, "x-polling-merchants: $merchantId");
        }

        curl_setopt_array(self::$curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
            CURLOPT_HTTPHEADER => $cabecalhoSolicitacao,
        ]);
          
        $response = curl_exec(self::$curl);
        $httpcode = curl_getinfo(self::$curl)['http_code'];
        curl_close(self::$curl);

        if(substr($httpcode, 0, 1) == 5) {
            Request::salvarErro($url, "GET", "$httpcode", $response);
        }

        return (object)["http_code"=> $httpcode, "body"=> $encode ? json_decode($response) : $response];
    }

    private static function salvarErro(string $url, string $metodo, string $codigoHttp, string $body) {
        $abrirArquivo = fopen("../../integracao/ifood/log_erros.txt", "a");
        $dataAtual = date("d/m/Y H:i:s");
        fwrite($abrirArquivo, "Data: $dataAtual\r\nEndpoint: $url\r\nMetodo: $metodo\r\nCodigo HTTP: $codigoHttp\r\nCorpo: $body\r\n\r\n");
        fclose($abrirArquivo);
    }

    public static function delete(string $url, string $token) {
        self::$curl = curl_init();
        curl_setopt_array(self::$curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'DELETE',
            CURLOPT_HTTPHEADER => array(
              "Authorization: Bearer $token"
            ),
        ]);
          
        $response = curl_exec(self::$curl);
        $httpcode = curl_getinfo(self::$curl)['http_code'];

        if(substr($httpcode, 0, 1) == 5) {
            Request::salvarErro($url, "GET", "$httpcode", $response);
        }

        return (object)["http_code"=> $httpcode, "body"=> json_decode($response) ?? []];
    }
}