<?php
namespace LojaVirtual\App\Config;

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\ErroLoja;

class Empresa
{
    private static $instancia;
    private static $site;

    private static function buscarDados(array $colunas, $utf8 = false)
    {
        try {
            $colunas = implode(", ", $colunas);
            $getDadosEmpresa = BancoDados::consultar(
                "SELECT $colunas FROM unidade WHERE Sede = ? ORDER BY Id",
                ["Matriz"],
                $utf8
            );

            if($getDadosEmpresa->rowCount() < 1) {
                throw new \Exception("Empresa no encontrada.");
            }
    
            return $getDadosEmpresa->fetchObject();
        } catch (\Throwable $th) {
            new ErroLoja($th);
        }
    }

    public static function dados(?array $colunas = null, $utf8 = false)
    {
        if (self::$instancia === null) {
            self::$instancia = self::buscarDados([
                "Id", "Titulo", "Cnpj", "Estado", "Sede", "Logo_Site", "Favicon_Site", "Site", "Tabela_Pedidos_Site", "Id_Tabela_Preco_Site",
                "Meta_Desconto_Site", "Valor_Minimo_Desconto_Site", "Percentual_Desconto_Site", "Obrigatorio_Login_Pedido_Site", "Plataforma_Pagamento_Site",
                "Tipo_Site", "Cep"
            ]);
        }

        return $colunas !== null ? (object) self::buscarDados($colunas, $utf8) : (object) self::$instancia;
    }

    public static function tabelaPreco()
    {
        $idTabelaPreco = self::dados()->Id_Tabela_Preco_Site ?? 1;
        $getTabelaPreco = BancoDados::consultar("SELECT Valor FROM produto_tabela WHERE Id = '$idTabelaPreco'");
        $tabelaPreco = $getTabelaPreco->fetchObject()->Valor ?? 1;
        
        return (int) $tabelaPreco;
    }

    public static function venderEstoqueNegativo()
    {
        $colunaEstoque = self::dados()->Tabela_Pedidos_Site == "Pr-Venda" ? "Vender_Estoque_Negativo_Pre" : "Vender_Estoque_Negativo";
        $getConfigEstoque = BancoDados::consultar("SELECT $colunaEstoque Vender_Negativo FROM opcao");
        $venderNegativo = $getConfigEstoque->fetchObject()->Vender_Negativo;
        
        return $venderNegativo === "Sim";
    }

    public static function setHost(string $host)
    {
        self::$site = $host == "localhost" ? "http://localhost/loja_roupa" : "https://$host";
    }
}