<?php
namespace LojaVirtual\App\Config\Http;
class ClientRequest
{
    private static $curlOptions = [
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
    ];

    public static function get(string $url, ?array $headers = null)
    {
        $curl = curl_init();
        $curlOptions = self::$curlOptions;
        $curlOptions[CURLOPT_URL] = $url;
        $curlOptions[CURLOPT_CUSTOMREQUEST] = 'GET';
        if($headers) {
            $curlOptions[CURLOPT_HTTPHEADER]=  $headers;
        }

        curl_setopt_array($curl, $curlOptions);
        $response = curl_exec($curl);
        $responseHeaders = curl_getinfo($curl);

        curl_close($curl);
        return (object)[
            "headers"=> $responseHeaders,
            "body"=> json_decode($response) ?? $response
        ];
    }

    public static function post(string $url, array $headers, string $dados)
    {
        $curl = curl_init();
        $curlOptions = self::$curlOptions;
        $curlOptions[CURLOPT_URL] = $url;
        $curlOptions[CURLOPT_CUSTOMREQUEST] = 'POST';
        $curlOptions[CURLOPT_POSTFIELDS] = $dados;
        if(sizeof($headers) > 0) {
            $curlOptions[CURLOPT_HTTPHEADER] =  $headers;
        }

        curl_setopt_array($curl, $curlOptions);
        $response = curl_exec($curl);
        $responseHeaders = curl_getinfo($curl);
        curl_close($curl);
        return (object)[
            "headers"=> (object)$responseHeaders,
            "body"=> json_decode($response) ?? $response
        ];
    }

    public static function put(string $url, array $headers, string $dados)
    {
        $curl = curl_init();
        $curlOptions = self::$curlOptions;
        $curlOptions[CURLOPT_URL] = $url;
        $curlOptions[CURLOPT_CUSTOMREQUEST] = 'PUT';
        $curlOptions[CURLOPT_POSTFIELDS] = $dados;
        $curlOptions[CURLOPT_HTTPHEADER] =  $headers;

        curl_setopt_array($curl, $curlOptions);
        $response = curl_exec($curl);
        $responseHeaders = curl_getinfo($curl);
        curl_close($curl);
        return (object)[
            "headers"=> (object)$responseHeaders,
            "body"=> json_decode($response) ?? $response
        ];
    }

    public static function patch(string $url, array $headers, string $dados)
    {
        $curl = curl_init();
        $curlOptions = self::$curlOptions;
        $curlOptions[CURLOPT_URL] = $url;
        $curlOptions[CURLOPT_CUSTOMREQUEST] = 'PATCH';
        $curlOptions[CURLOPT_POSTFIELDS] = $dados;
        $curlOptions[CURLOPT_HTTPHEADER] =  $headers;

        curl_setopt_array($curl, $curlOptions);
        $response = curl_exec($curl);
        $responseHeaders = curl_getinfo($curl);
        curl_close($curl);
        return (object)[
            "headers"=> (object)$responseHeaders,
            "body"=> json_decode($response) ?? $response
        ];
    }

    public static function delete(string $url, array $headers)
    {
        $curl = curl_init();
        $curlOptions = self::$curlOptions;
        $curlOptions[CURLOPT_URL] = $url;
        $curlOptions[CURLOPT_CUSTOMREQUEST] = 'DELETE';
        $curlOptions[CURLOPT_HTTPHEADER] =  $headers;

        curl_setopt_array($curl, $curlOptions);
        $response = curl_exec($curl);
        $responseHeaders = curl_getinfo($curl);
        curl_close($curl);
        return (object)[
            "headers"=> (object)$responseHeaders,
            "body"=> json_decode($response) ?? $response
        ];
    }
}