<?php
namespace LojaVirtual\App\Config;

use Closure;

class Rota
{
    private $url;
    private $parameters;
    private $queryParameters;
    private $headers;
    private $body;
    private $files;
    private static $apiPrefix = "/api";
    private $qtyPathParameters = 0;

    public function __construct()
    {
        $this->setRoute();
        $this->headers = (object)$_SERVER;
        $this->files = (object)$_FILES;
        $this->body = (object)$_POST;
        
        if(isset($this->headers->CONTENT_TYPE) && $this->headers->CONTENT_TYPE == "application/json") {
            $this->body = json_decode(file_get_contents("php://input"));
        }

        $_SERVER = [];
        $_FILES = [];
        $_POST = [];
        $_GET = [];
    }

    private static function mountRoute()
    {
        $route = (object)[
            "url"=> "",
            "pathParameters"=> []
        ];

        $getUrl = isset($_GET['url']) ? explode("/", addslashes($_GET['url'])) : null;
        if ($getUrl) {
            for ($i = 0; $i < sizeof($getUrl); $i++) {
                $isNumber = strlen(preg_replace("/[^0-9]/", "", $getUrl[$i])) > 0 ? true : false;
                if ($isNumber) {
                    array_push($route->pathParameters, $getUrl[$i]);
                } else {
                    $route->url .= "/$getUrl[$i]";
                }
            }
        }
        return $route;
    }
    private function setRoute()
    {
        $mountRoute = Rota::mountRoute();
        $this->url = $mountRoute->url;
        $this->parameters = $mountRoute->pathParameters;
        $this->setQueryParams();
        $this->qtyPathParameters = sizeof($mountRoute->pathParameters);
    }

    public function getRoute()
    {
        return $this->url;
    }

    private function mountParameters(string $url)
    {
        $explodeUrl = explode('/', $url);
        $qtyParameters = 0;

        foreach($explodeUrl as $urlPath) {
            if(preg_match("/[:]+[{]+[a-zA-Z]+[}]/", $urlPath) && isset($this->parameters[$qtyParameters])) {
                $indice = str_replace([":", "{", "}"],  "", $urlPath);
                $this->parameters[$indice] = $this->parameters[$qtyParameters];
                unset($this->parameters[$qtyParameters]);
                $qtyParameters++;
            }
        }
        $this->parameters = (object)$this->parameters;
    }

    public function getParameters()
    {
        return $this->parameters;
    }

    private function setQueryParams()
    {
        unset($_GET["url"]);
        if(sizeof($_GET) > 0) {
            foreach($_GET as $key => $value) {
                $this->queryParameters[$key] = $value;
            }
        } else {
            $this->queryParameters = [];
        }
    }

    public function getQueryParameters()
    {
        return (object)$this->queryParameters;
    }

    public function header()
    {
        return $this->headers;
    }

    public function body()
    {

        return $this->body;
    }

    public function files()
    {
        return $this->files;
    }

    public function get(string $url, ?Closure $middleware, callable $callback)
    {
        $this->verifyRequest("GET", $url, $middleware, $callback);
    }

    public function post(string $url, ?Closure $middleware, callable $callback)
    {
        $this->verifyRequest("POST", $url, $middleware, $callback);
    }

    public function put(string $url, Closure $middleware, callable $callback)
    {
        $this->verifyRequest("PUT", $url, $middleware, $callback);
    }

    public function patch(string $url, Closure $middleware, callable $callback)
    {
        $this->verifyRequest("PATCH", $url, $middleware, $callback);
    }

    public function delete(string $url, Closure $middleware, callable $callback)
    {
        $this->verifyRequest("DELETE", $url, $middleware, $callback);
    }

    private function verifyRequest(string $method, string $url, ?Closure $middleware, callable $callback)
    {
        $acceptedMethods = ["GET", "HEAD", "OPTIONS", "PUT", "POST", "DELETE", "PATCH"];

        if(!in_array($method, $acceptedMethods)) {
            throw new \Exception("Mtodo no permitido.");
        }
        $originalUrl = $url;
        $url = preg_replace("/[\/]+[:]+[{]+[a-zA-Z]+[}]/", "", $url);
        $qtyPathParameters = preg_match_all("/[:]+[{]+[a-zA-Z]+[}]/", $originalUrl);

        if($this->headers->REQUEST_METHOD == $method && $this->url == $this::$apiPrefix.$url && $qtyPathParameters == $this->qtyPathParameters) {
            if(preg_match("/[:]+[{]+[a-zA-Z]+[}]/", $originalUrl)) {
                $this->mountParameters($originalUrl);
            }
            if(is_callable($middleware)) {
                $middleware($this);
            }
            $callback($this);
        }
    }
}