<?php
namespace LojaVirtual\App\Controller;

use LojaVirtual\App\Config\Database\DatabaseQuery;
use LojaVirtual\App\Config\Database\Where;
use LojaVirtual\App\Config\Rota;
use LojaVirtual\App\Model\Cliente;
use LojaVirtual\App\Model\Type\Customer\Endereco;

class ClienteController
{
    public static function criarCliente(Rota $rota)
    {
        $body = $rota->body();
        $titulo = codificarString(addslashes($body->titulo), "UTF-8", "ISO-8859-1");
        $razaoSocial = codificarString(addslashes($body->razaoSocial), "UTF-8", "ISO-8859-1");
        $telefone = addslashes($body->telefone);
        $cpfCnpj = str_replace("/[^0-9]/", "", addslashes($body->cpfCnpj));
        $email = addslashes($body->email);
        $natureza = addslashes($body->natureza);
        $senha = addslashes($body->senha);

        $clienteExiste = Cliente::buscar(
            new DatabaseQuery(
                [
                    new Where("Cpf_Cnpj", $cpfCnpj)
                ]
            )
        );

        if($clienteExiste) {
            header("Content-Type: application/json");
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString(($natureza == "PF" ? "CPF" : "CNPJ")." j cadastrado", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }

        $cliente = new Cliente(
            $titulo, 
            $razaoSocial, 
            $cpfCnpj, 
            $natureza, 
            $email, 
            $telefone, 
            $senha,
            289
        );
        $salvarCliente = $cliente->salvar();

        if($salvarCliente !== false) {
            header("HTTP/1.1 201 Created");
            echo json_encode([
                "id"=> $cliente->id
            ]);
        } else {
            header("Content-Type: application/json");
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("No foi possvel realizar o cadastro", "ISO-8859-1", "UTF-8")
            ]);
        }
    }

    public static function login(Rota $rota)
    {
        $body = $rota->body();
        $cpfCnpj = str_replace("/[^0-9]/", "", addslashes($body->cpfCnpj));
        $senha = addslashes($body->senha);

        $cliente = Cliente::buscar(
            new DatabaseQuery([
                new Where("Cpf_Cnpj", $cpfCnpj),
                new Where("Senha_Site", $senha)
            ])
        );

        header("Content-Type: application/json");

        if(isset($cliente->cpfCnpj)) {
            unset($cliente->senha);
    
            echo json_encode($cliente);
        } else {
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("CPF/CNPJ ou senha invlidos", "ISO-8859-1", "UTF-8")
            ]);
        }
    }

    public static function cadastrarEndereco(Rota $rota)
    {
        $body = $rota->body();
        $idCliente = $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $contato = addslashes($body->contato);
        $telefone = preg_replace("/[^0-9]/", "", addslashes($body->telefone));
        $cep = preg_replace("/[^0-9]/", "", addslashes($body->cep));
        $logradouro = addslashes($body->logradouro);
        $numero = addslashes($body->numero);
        $complemento = addslashes($body->complemento);
        $bairro = addslashes($body->bairro);
        $cidade = addslashes($body->cidade);
        $estado = addslashes($body->estado);
        $ibge = addslashes($body->ibge);

        $endereco = new Endereco(
            $contato,
            $telefone,
            $estado,
            $cidade,
            $bairro,
            $logradouro,
            $numero,
            $cep,
            $ibge,
            $complemento
        );
        $salvarEndereco = $endereco->salvar($idCliente);

        header("Content-Type: application/json");
        if($salvarEndereco == false) {
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("No foi possvel cadastrar o endereo", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }

        $enderecos = Cliente::enderecos($idCliente);
        echo json_encode([
            "status"=> "ok",
            "enderecos"=> $enderecos
        ]);
    }

    public static function atualizarEndereco(Rota $rota)
    {
        header("Content-Type: application/json");

        if(!isset($rota->getParameters()->idEndereco)) {
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("ID do endereo no informado", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }
        $body = $rota->body();
        $idCliente = $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $idEndereco = addslashes($rota->getParameters()->idEndereco);
        $contato = addslashes($body->contato);
        $telefone = preg_replace("/[^0-9]/", "", addslashes($body->telefone));
        $cep = preg_replace("/[^0-9]/", "", addslashes($body->cep));
        $logradouro = addslashes($body->logradouro);
        $numero = addslashes($body->numero);
        $complemento = addslashes($body->complemento);
        $bairro = addslashes($body->bairro);
        $cidade = addslashes($body->cidade);
        $estado = addslashes($body->estado);
        $ibge = addslashes($body->ibge);

        $endereco = new Endereco(
            $contato,
            $telefone,
            $estado,
            $cidade,
            $bairro,
            $logradouro,
            $numero,
            $cep,
            $ibge,
            $complemento
        );
        $endereco->id = $idEndereco;
        $salvarEndereco = $endereco->atualizar($idCliente);

        if($salvarEndereco == false) {
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("No foi possvel atualizar o endereo", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }

        $enderecos = Cliente::enderecos($idCliente);
        echo json_encode([
            "status"=> "ok",
            "enderecos"=> $enderecos
        ]);
    }

    public static function atualizarEnderecoPadrao(Rota $rota)
    {
        if(!isset($rota->getParameters()->idEndereco) || !isset($rota->body()->padrao)) {
            header("Content-Type: application/json");
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("Parmetro obrigatrio no informado", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }
        $body = $rota->body();
        $idCliente = $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $idEndereco = addslashes($rota->getParameters()->idEndereco);
        $padrao = addslashes($body->padrao);
        $atualizarEnderecoPadrao = Endereco::atualizarPadrao($idCliente, (int)$idEndereco, $padrao);

        if($atualizarEnderecoPadrao == false) {
            header("Content-Type: application/json");
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("No foi possvel atualizar o endereo", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }
        header("HTTP/1.1 204 No Content");
    }

    public static function enderecosCliente(Rota $rota)
    {
        $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $enderecos = Cliente::enderecos($idCliente);

        header("Content-Type: application/json");
        echo json_encode($enderecos);
    }

    public static function detalhesEndereco(Rota $rota)
    {
        header("Content-Type: application/json");
        
        if(!isset($rota->getParameters()->idEndereco)) {
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("ID do endereo no informado", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }

        $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $idEndereco = $rota->getParameters()->idEndereco;
        [$endereco] = Cliente::enderecos($idCliente, (int)$idEndereco);

        echo json_encode($endereco);
    }

    public static function excluirEndereco(Rota $rota)
    {
        if(!isset($rota->getParameters()->idEndereco)) {
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("ID do endereo no informado", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }

        $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $idEndereco = $rota->getParameters()->idEndereco;
        $excluirEndereco = Endereco::excluir($idCliente, (int)$idEndereco);

        if($excluirEndereco == false) {
            header("Content-Type: application/json");
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("No foi possvel excluir o endereo", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }
        header("HTTP/1.1 204 No Content");
    }

    public static function enderecoPadrao(Rota $rota)
    {
        $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $enderecoPadrao = Cliente::enderecoPadrao($idCliente);
        header("Content-Type: application/json");
        echo json_encode($enderecoPadrao ?? ["status"=> "erro", "mensagem"=> "Nenhum endereco encontrado"]);
    }

    public static function pedidosCliente(Rota $rota) {
        $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $pedidos = Cliente::pedidos($idCliente);
        header("Content-Type: application/json");
        echo json_encode($pedidos);
    }

    public static function detalhesPedido(Rota $rota)
    {
        $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
        $idPedido = $rota->getParameters()->idPedido;
        [$pedido] = Cliente::pedidos($idCliente, $idPedido);

        header("Content-Type: application/json");
        
        if(empty($pedido)) {
            header("HTTP/1.1 404 Not Found");
            echo json_encode(["status"=> "erro", "mensagem"=> codificarString("Pedido no encontrado", "ISO-8859-1", "UTF-8")]);
            exit;
        }
        
        echo json_encode($pedido);
    }
}