<?php
namespace LojaVirtual\App\Controller;

use LojaVirtual\App\Config\Empresa;
use LojaVirtual\App\Config\Rota;
use LojaVirtual\App\Model\Pagamento\PagamentoMercadoPago;
use LojaVirtual\App\Model\TipoPagamento;
use MercadoPago\Src\Config\Autenticacao as AutenticacaoMercadoPago;

class PagamentoController
{
    public static function tiposPagamento()
    {
        $tiposPagamento = TipoPagamento::buscarTodos();
        header('Content-Type: application/json');
        echo json_encode($tiposPagamento);
    }
    
    public static function tokenAutenticacaoPagamento()
    {
        $tokenAutenticacao = AutenticacaoMercadoPago::getTokenPagamento();
        $maximoParcelas = 6;

        header('Content-Type: application/json');
        echo json_encode([
            "tokenAutenticacao"=> $tokenAutenticacao,
            "maximoParcelas"=> $maximoParcelas
        ]);
    }

    public static function atualizacaoPagamentoMP(Rota $rota)
    {
        $body = $rota->body();
        $assinaturaRequest = $rota->header()->HTTP_X_SIGNATURE;
        $explodeAssinatura = explode(",", $assinaturaRequest);
        $assinaturaSecreta = Empresa::dados(["Token_Webhook_Pagamento"])->Token_Webhook_Pagamento;
        $queryParametros = $rota->getQueryParameters();
        $dataId = $queryParametros->data_id;
        $idRequest = $rota->header()->HTTP_X_REQUEST_ID;

        $ts = null;
        $hash = null;

        foreach ($explodeAssinatura as $part) {
            $keyValue = explode('=', $part, 2);
            
            if (sizeof($keyValue) == 2) {
                $key = trim($keyValue[0]);
                $value = trim($keyValue[1]);
                if ($key === "ts") {
                    $ts = $value;
                } elseif ($key === "v1") {
                    $hash = $value;
                }
            }
        }
        $hashAssinatura = hash_hmac('sha256', "id:$dataId;request-id:$idRequest;ts:$ts;", $assinaturaSecreta);
        if($hashAssinatura != $hash) {
            header("HTTP/1.1 401 Unauthorized");
            exit;
        }

        if($body->type = "payment" && $body->action == "payment.updated") {
            $idPagamento = addslashes($body->data->id);
            $detalhesPagamento = PagamentoMercadoPago::detalhes($idPagamento);

            if(isset($detalhesPagamento->id)) {
                PagamentoMercadoPago::atualizarStatusInterno($detalhesPagamento);
            }
        }
    }
}