<?php
namespace LojaVirtual\App\Controller;

use LojaVirtual\App\Config\Database\DatabaseQuery;
use LojaVirtual\App\Config\Database\Where;
use LojaVirtual\App\Config\Empresa;
use LojaVirtual\App\Config\ErroLoja;
use LojaVirtual\App\Config\Rota;
use LojaVirtual\App\Model\Cliente;
use LojaVirtual\App\Model\Frete\Frete;
use LojaVirtual\App\Model\Pedido;
use LojaVirtual\App\Model\Produto;
use LojaVirtual\App\Model\TipoPagamento;
use LojaVirtual\App\Model\Type\Customer\Endereco;
use LojaVirtual\App\Model\Type\Order\Entrega;

class PedidoController
{
    public static function cadastrarPedido(Rota $rota)
    {
        try {
            $empresa = Empresa::dados();
            $body = $rota->body();
            $cliente = null;
            $idEnderecoEntrega = 0;

            if(isset($rota->header()->HTTP_CUSTOM_AUTH)) {
                $idCliente = addslashes($rota->header()->HTTP_CUSTOM_AUTH);
                $cliente = Cliente::buscar(
                    new DatabaseQuery(
                        [
                            new Where("Id", $idCliente)
                        ]
                    )
                );
                
                if(!$cliente) {
                    header('Content-Type: application/json');
                    echo json_encode([
                        "status"=>  "erro",
                        "mensagem"=> codificarString("Usurio no encontrado", "ISO-8859-1", "UTF-8"),
                    ]);
                    exit;
                }
                $idEnderecoEntrega = (int)addslashes($body->endereco->id);
                $buscarEndereco = Endereco::buscar(
                    new DatabaseQuery(
                        [
                            new Where("Id_Cliente", $idCliente),
                            new Where("Id", $idEnderecoEntrega)
                        ]
                    )
                );
                if(!isset($buscarEndereco[0])) {
                    header('Content-Type: application/json');
                    echo json_encode([
                        "status"=>  "erro",
                        "mensagem"=> codificarString("Endereo invlido", "ISO-8859-1", "UTF-8"),
                    ]);
                    exit;
                }
            } else {
                $titulo = addslashes($body->titulo);
                $razaoSocial = "";
                $cpfCnpj = preg_replace("/[^0-9]/", "", addslashes($body->cpfCnpj));
                $cliente = Cliente::buscar(
                    new DatabaseQuery(
                        [
                            new Where("Cpf_Cnpj", $cpfCnpj)
                        ]
                    )
                );

                if(!$cliente) {
                    $natureza = strlen($cpfCnpj) == 11 ? "PF" : "PJ";
                    $email = addslashes($body->email);
                    $telefone = preg_replace("/[^0-9]/", "", addslashes($body->telefone));
                    $senha = addslashes($body->senha);
                    $endereco = new Endereco(
                        addslashes(codificarString($body->endereco->entrega->contato, "UTF-8", "ISO-8859-1")),
                        addslashes($body->endereco->entrega->telefoneContato),
                        addslashes($body->endereco->entrega->uf),
                        addslashes(codificarString($body->endereco->entrega->cidade, "UTF-8", "ISO-8859-1")),
                        addslashes(codificarString($body->endereco->entrega->bairro, "UTF-8", "ISO-8859-1")),
                        addslashes(codificarString($body->endereco->entrega->logradouro, "UTF-8", "ISO-8859-1")),
                        addslashes($body->endereco->entrega->numero),
                        addslashes($body->endereco->entrega->cep),
                        addslashes($body->endereco->entrega->ibge),
                        addslashes(codificarString($body->endereco->entrega->complemento, "UTF-8", "ISO-8859-1")),
                        "Sim"
                    );

                    $idCfop = $endereco->uf == $empresa->Estado ? 289 : 440;

                    $cliente = new Cliente(
                        codificarString($titulo, "UTF-8", "ISO-8859-1"),
                        codificarString($razaoSocial, "UTF-8", "ISO-8859-1"),
                        $cpfCnpj,
                        $natureza,
                        $email,
                        $telefone,
                        $senha,
                        $idCfop
                    );
                    $cadastrarCliente = $cliente->salvar();
                    $cadastrarEndereco = $endereco->salvar($cliente->id);
                    $idEnderecoEntrega = (int)$cadastrarEndereco;

                    if(!$cadastrarCliente) {
                        header('Content-Type: application/json');
                        echo json_encode([
                            "status"=>  "erro",
                            "mensagem"=> codificarString("No foi possvel concluir seu pedido", "ISO-8859-1", "UTF-8"),
                        ]);
                        exit;
                    }
                }
            }

            $empresa = Empresa::dados();
            $produtos = self::buscarProdutosPedido($body->produtos);
            $valorTotal = 0;
            $valorPago = 0.00;
            $status = "Aberto";
            $pagamento = $empresa->Tipo_Site == "Catlogo" ? (int)addslashes($body->tipoPagamento) : $body->pagamento;
            $valorEntrega = 0;
            $desconto = 0;

            if($empresa->Tipo_Site == "Catlogo") {
                $valorEntrega = pedido::calcularValorEntrega($idEnderecoEntrega);
            } else {
                $idFrete = addslashes($body->frete->id);
                $valorEntrega = Frete::getValorCotacao((int)$idFrete, $cliente->cpfCnpj) ?? 0;
                $explodeNomeCliente = explode(" ", $cliente->titulo);
                $pagamento->payer->first_name = $explodeNomeCliente[0];
                $pagamento->payer->last_name = $explodeNomeCliente[1];

                $explodeUrl = explode("/", $rota->header()->PHP_SELF);
                $hostNotificacao = $rota->header()->HTTP_HOST == 'localhost' ? "ponttiweb.com.br/lojavirtual" : "{$rota->header()->HTTP_HOST}/$explodeUrl[1]";
                $urlNotificacao = "https://$hostNotificacao/integracao/loja_virtual/api/pagamentos/mercado_pago/webhooks";
                $pagamento->notification_url = $urlNotificacao;
            }

            foreach($produtos as $produto) {
                $valorTotal += $produto->preco * $produto->quantidade;
                if($empresa->Tipo_Site == "Loja Virtual") {
                    $pagamento->additional_info->items[] = [
                        "quantity"=> $produto->quantidade,
                        "unit_price"=> $produto->preco
                    ];
                }
            }
            
            if($empresa->Meta_Desconto_Site !== "Nenhum" && $empresa->Valor_Minimo_Desconto_Site > 0 && $valorTotal >= $empresa->Valor_Minimo_Desconto_Site) {
                if($empresa->Meta_Desconto_Site == "Pedido") {
                    $desconto = $empresa->Percentual_Desconto_Site;
                } else {
                    if($valorEntrega > 0) {
                        $valorEntrega -=  $valorEntrega * ($empresa->Percentual_Desconto_Site / 100);
                        $valorEntrega = $valorEntrega < 0 ? 0 : $valorEntrega;
                    }
                }
            }

            if(isset($body->pagamento->transaction_amount)) {
                $valorTotalPedido = $empresa->Meta_Desconto_Site == "Pedido" ? $valorTotal - $desconto : $valorTotal;
                $valorTotalPedido += $valorEntrega;

                if($body->pagamento->transaction_amount < $valorTotalPedido) {
                    header('Content-Type: application/json');
                    echo json_encode([
                        "status"=>  "erro",
                        "mensagem"=> codificarString("O valor do pagamento  menor que o valor total do pedido", "ISO-8859-1", "UTF-8")
                    ]);
                    exit;
                }
            }
            $entrega = new Entrega(
                $idEnderecoEntrega,
                addslashes($body->observacao)
            );
            $valorEntrega = (float)number_format($valorEntrega, 2, '.', '');
            
            $pedido = new Pedido(
                $valorTotal,
                $valorPago,
                $status,
                $produtos,
                $desconto,
                $entrega,
                $empresa->Id,
                $cliente->getIdVendedor(),
                $cliente->getIdVendedor(),
                $cliente->id,
                $pagamento,
                addslashes(codificarString($body->observacao, "UTF-8", "ISO-8859-1")),
                $cliente->idCfop,
                $valorEntrega,
                date("Y-m-d H:i:s")
            );
            $cadastrarPedido = $pedido->salvar();

            if(!$cadastrarPedido) {
                header('Content-Type: application/json');
                echo json_encode([
                    "status"=>  "erro",
                    "mensagem"=> codificarString("No foi possvel concluir seu pedido", "ISO-8859-1", "UTF-8"),
                ]);
                exit;
            }
            header("HTTP/1.1 201 Created");
            header('Content-Type: application/json');

            $response = [
                "id"=> $cadastrarPedido
            ];
            if($pedido->transacao) {
                $response['pagamentoPendente'] = $pedido->transacao != null;
            }
            echo json_encode($response);
        } catch (\Throwable $th) {
           new ErroLoja($th);
        }
    }

    private static function buscarProdutosPedido(array $produtos)
    {
        $valoresProdutos = [];
        foreach($produtos as $indice => $produto) {
            $whereProduto = [];

            if(isset($produto->tamanho)) {
                $whereProduto[] = new Where("produto.Grade_Tamanho", addslashes($produto->tamanho));
            }
            if(isset($produto->cor)) {
                $whereProduto[] = new Where("produto.Grade_Cor", addslashes($produto->cor));
            }
            if(isset($produto->tipo)) {
                $whereProduto[] = new Where("produto.Grade_Genero", addslashes($produto->tipo));
            }
            if(isset($produto->especificacao)) {
                $whereProduto[] = new Where("produto.Grade_Especificacao", addslashes($produto->especificacao));
            }

            $colunaIdentificacao = sizeof($whereProduto) > 0 ? "Id_Produto_Pai" : "Id";
            $whereProduto[] = new Where("produto.{$colunaIdentificacao}", addslashes($produto->id));
            $valorProduto = Produto::buscarTodos(
                new DatabaseQuery(
                    $whereProduto
                ),
                ["produto.Id", "produto.Preco", "produto.Codigo", "produto.Titulo"]
            );

            if(isset($valorProduto[0])) {
                $valoresProdutos[] = (object)[
                    "id"=> $valorProduto[0]->id,
                    "preco"=> $valorProduto[0]->preco,
                    "quantidade"=> $produto->quantidade
                ];
            }
        }
        return $valoresProdutos;
    }

    public static function tiposPagamento()
    {
        $tiposPagamento = TipoPagamento::buscarTodos();
        header('Content-Type: application/json');
        echo json_encode($tiposPagamento);
    }

    public static function valoresFrete(Rota $rota)
    {
        $body = $rota->body();
        $cepDestino = preg_replace("/[^0-9]/", "", $body->cepDestino);
        $produtosCarrinho = $body->produtos;
        $dadosEmpresa = Empresa::dados(["Cep"]);
        $cepOrigem = preg_replace("/[^0-9]/", "", $dadosEmpresa->Cep);
        $pesoBrutoProdutos = 0;
        $dimensoesPacote = [
            "largura"=> 0,
            "altura"=> 0,
            "comprimento"=> 0,
            "peso"=> 0
        ];

        foreach($produtosCarrinho as $produto) {
            $dimensoesProduto = null;

            if(isset($produto->tamanho) || isset($produto->cor) || isset($produto->tipo) || isset($produto->especificacao)) {
                $where = [
                    new Where("Id_Produto_Pai", $produto->id)
                ];
                if(isset($produto->tamanho)) {
                    $where[] = new Where("Grade_Tamanho", $produto->tamanho);
                }
                if(isset($produto->cor)) {
                    $where[] = new Where("Grade_Cor", $produto->cor);
                }
                if(isset($produto->tipo)) {
                    $where[] = new Where("Grade_Genero", $produto->tipo);
                }
                if(isset($produto->especificacao)) {
                    $where[] = new Where("Grade_Especificacao", $produto->especificacao);
                }
                [$dimensoesProduto] = Produto::dimensoes(
                    new DatabaseQuery($where)
                );
            } else {
                [$dimensoesProduto] = Produto::dimensoes(
                    new DatabaseQuery(
                        [
                            new Where("Id", $produto->id)
                        ]
                    )
                );
            }
            
            if($dimensoesProduto) {
                $dimensoesPacote['peso'] += $produto->quantidade * $dimensoesProduto->peso;
                $dimensoesPacote['largura'] = max($dimensoesPacote['largura'], $dimensoesProduto->largura * 100);
                $dimensoesPacote['altura'] = max($dimensoesPacote['altura'], $dimensoesProduto->altura * 100);
                $dimensoesPacote['comprimento'] = max($dimensoesPacote['comprimento'], $dimensoesProduto->comprimento * 100);
            }
        }

        $dadosFrete = Frete::calcular($cepOrigem, $cepDestino, $dimensoesPacote);
        $plataforma = "SuperFrete";
        $cpfCnpj = isset($body->cpfCnpj) ? addslashes($body->cpfCnpj) : null;

        if(isset($dadosFrete->errors)) {
            header('Content-Type: application/json');
            echo json_encode($dadosFrete);
            exit;
        }

        if(!$cpfCnpj) {
            $idCliente = addslashes($body->idCliente);
            $cliente = Cliente::buscar(new DatabaseQuery(
                [
                    new Where("Id", $idCliente)
                ]
            ));
            $cpfCnpj = $cliente->cpfCnpj;
        }
        $cpfCnpj = preg_replace("/[^0-9]/", "", $cpfCnpj);
        header('Content-Type: application/json');
        
        if(strlen($cpfCnpj) < 1) {
            echo json_encode([
                "status"=>  "erro",
                "mensagem"=> codificarString("CPF/CNPJ invlido", "ISO-8859-1", "UTF-8"),
            ]);
            exit;
        }
        Frete::excluirCotacoesCliente($cpfCnpj);
        foreach($dadosFrete as $frete) {
            if(isset($frete->price)) {
                Frete::salvarCotacao($frete->id, $frete->price, $plataforma, $cpfCnpj);
            }
        }
        echo json_encode($dadosFrete);
    }
}