<?php
namespace LojaVirtual\App\Controller;

use LojaVirtual\App\Config\Database\DatabaseQuery;
use LojaVirtual\App\Config\Database\Having;
use LojaVirtual\App\Config\Database\Limit;
use LojaVirtual\App\Config\Database\OrderBy;
use LojaVirtual\App\Config\Database\Where;
use LojaVirtual\App\Config\Empresa;
use LojaVirtual\App\Config\Rota;
use LojaVirtual\App\Model\Categoria;
use LojaVirtual\App\Model\Produto;
use LojaVirtual\App\Model\VariacaoProduto;

class ProdutoController
{
    public static function listarProdutos(Rota $rota)
    {
        $qtdProdutosEncontrados = 0;
        $response = [];

        $queryParametros = $rota->getQueryParameters();
        $venderEstoqueNegativo = Empresa::venderEstoqueNegativo();
        $tabelaPreco = Empresa::tabelaPreco();
        $paginaAtual = $queryParametros->pagina ?? 1;
        $produtosPorPagina = 20;
        $limite = $paginaAtual == 1 ? 0 : ($paginaAtual - 1) * $produtosPorPagina;
        $ordens = Produto::filtrosOrdenacao();
        $chavesOrdenacao = array_keys($ordens);
        $ordem = isset($queryParametros->ordenar) ? addslashes($queryParametros->ordenar) : $ordens[$chavesOrdenacao[0]];
        $ordenacao = [];

        if(in_array($ordem, $chavesOrdenacao)) {
            switch($ordem) {
                case "preco_crescente":
                    $ordenacao[] = new OrderBy("produto.Preco{$tabelaPreco}", "ASC");
                    break;
                case "preco_decrescente":
                    $ordenacao[] = new OrderBy("produto.Preco{$tabelaPreco}", "DESC");
                    break;
                case "produto_mais_recente":
                    $ordenacao[] = new OrderBy("produto.Data_Cadastro", "DESC");
                    break;
                case "produto_mais_antigo":
                    $ordenacao[] = new OrderBy("produto.Data_Cadastro", "ASC");
                    break;
            }
        }
        $opcoesConsulta = [
            "where"=> [],
            "order"=> $ordenacao,
            "limit"=> new Limit($limite, $produtosPorPagina),
            "groupBy"=> null,
            "having"=> [],
        ];

        if(!$venderEstoqueNegativo) {
            $opcoesConsulta['having'][] = new Having("Qtd_Estoque", ">", "0");
        }

        if(isset($queryParametros->categorias) && !empty($queryParametros->categorias)) {
            $nomesCategorias = str_replace(["-", "_"], [" ", "-"], addslashes($queryParametros->categorias));
            $nomesCategorias = codificarString($nomesCategorias, "ISO-8859-1", "UTF-8");
            $nomesCategorias = explode(',', addslashes($nomesCategorias));
            $categorias = Categoria::buscarTodas(
                new DatabaseQuery(
                    [
                        (new Where("Titulo"))->in($nomesCategorias)
                    ]
                )
            );
            $idsCategorias = array_column($categorias, 'id');
            $consulta = new DatabaseQuery(
                [
                    (new Where("produto.Id_Categoria_Produto"))->in($idsCategorias)
                ],
                $opcoesConsulta['order'],
                $opcoesConsulta['limit'],
                $opcoesConsulta['groupBy'],
                $opcoesConsulta['having']
            );
            $produtos = Produto::buscarTodos( $consulta);
            $qtdProdutosEncontrados = Produto::quantidadeProdutos($consulta);
            $response = [
                "produtos" => $produtos,
                "qtdProdutosEncontrados" => $qtdProdutosEncontrados,
                "produtosPorPagina"=> $produtosPorPagina
            ];
        } else {
            $consulta = new DatabaseQuery(
                $opcoesConsulta['where'],
                $opcoesConsulta['order'],
                $opcoesConsulta['limit'],
                $opcoesConsulta['groupBy'],
                $opcoesConsulta['having']
            );
            $produtos = Produto::buscarTodos($consulta);
            $qtdProdutosEncontrados = Produto::quantidadeProdutos($consulta);
            $response = [
                "produtos" => $produtos,
                "qtdProdutosEncontrados" => $qtdProdutosEncontrados,
                "produtosPorPagina"=> $produtosPorPagina
            ];
        }
        
        header("Content-Type: application/json");
        echo json_encode($response);
    }

    public static function listarProdutosDestaques()
    {
        $produtosDestaques = Produto::destaques();
        header("Content-Type: application/json");
        echo json_encode($produtosDestaques);
    }

    public static function detalhesProduto(Rota $rota)
    {
        header("Content-Type: application/json");

        if(!isset($rota->getParameters()->idProduto)) {
            echo json_encode([
                "status"=> "erro",
                "mensagem"=> codificarString("ID do produto no informado", "ISO-8859-1", "UTF-8")
            ]);
            exit;
        }
        $idProduto = addslashes($rota->getParameters()->idProduto);
        [$produto] = Produto::buscarTodos(
            new DatabaseQuery(
                [
                    new Where("produto.Id", $idProduto)
                ]
            )
        );
        $response = $produto ?? ["status"=> "erro", "mensagem"=> "Nenhum do produto encontrado"];
        echo json_encode($response);
    }

    public static function listarVariacoes(Rota $rota)
    {
        $idProduto = addslashes($rota->getParameters()->idProduto);
        $queryParametros = $rota->getQueryParameters();
        $gradeProduto = [
            "cor"=> "Grade_Cor",
            "tamanho"=> "Grade_Tamanho",
            "tipo"=> "Grade_Genero",
            "especificacao"=> "Grade_Especificacao"
        ];
        $parametrosGrade = (array)$queryParametros;
        $whereVariacoes = [
            new Where("produto.Id_Produto_Pai", $idProduto),
        ];
        if(sizeof($parametrosGrade) > 0) {
            foreach($parametrosGrade as $campo => $valor) {
                if(array_key_exists($campo, $gradeProduto)) {
                    $whereVariacoes[] = new Where("produto.{$gradeProduto[$campo]}", $valor);
                }
            }
        }
        $venderEstoqueNegativo = Empresa::venderEstoqueNegativo();
        $consulta = new DatabaseQuery(
            $whereVariacoes
        );
        if(!$venderEstoqueNegativo) {
            $consulta->setHaving([
                new Having("Qtd_Estoque", ">", 0)
            ]);
        }
        $variacoesProduto = VariacaoProduto::buscarTodas($consulta);

        header("Content-Type: application/json");
        echo json_encode($variacoesProduto);
    }

    public static function valoresProdutos(Rota $rota)
    {
        $produtosCarrinho = $rota->body();
        $whereVariacoes = [];
        $whereProdutos = [];
        $dadosEmpresa = Empresa::dados();

        foreach($produtosCarrinho as $produto) {
            if(isset($produto->tamanho) || isset($produto->cor) || isset($produto->tipo) || isset($produto->especificacao)) {
                $where = [
                    new Where("Id_Produto_Pai", $produto->id)
                ];
                if(isset($produto->tamanho)) {
                    $where[] = new Where("Grade_Tamanho", $produto->tamanho);
                }
                if(isset($produto->cor)) {
                    $where[] = new Where("Grade_Cor", $produto->cor);
                }
                if(isset($produto->tipo)) {
                    $where[] = new Where("Grade_Genero", $produto->tipo);
                }
                if(isset($produto->especificacao)) {
                    $where[] = new Where("Grade_Especificacao", $produto->especificacao);
                }
                $whereVariacoes[] = $where;
            } else {
                $whereProdutos[] = $produto->id;
            }
        }

        $variacoes = [];
        $colunasVariacoes = ["Id", "Id_Produto_Pai", "Titulo", "Preco", "Grade_Cor", "Grade_Tamanho", "Grade_Genero",
            "Grade_Especificacao", "Codigo", "Caminho_Foto", "Foto"];
        $variacoesProduto = sizeof($whereVariacoes) > 0 ? VariacaoProduto::buscarPrecos($whereVariacoes, $colunasVariacoes) : [];
        foreach($variacoesProduto as $variacao) {
            $codigoProduto = preg_replace("/[^0-9]/", "", $variacao->Codigo);
            $imagemProduto = "";

            if(!empty($variacao->Foto)) {
                if(file_exists("../../produto/fotos/$codigoProduto/original/$variacao->Foto")) {
                    $imagemProduto = "https://$variacao->Caminho_Foto/produto/fotos/$codigoProduto/original/$variacao->Foto";
                }
            }
            $variacoes[] = (object)[
                "id" => $variacao->Id,
                "idProdutoPai" => $variacao->Id_Produto_Pai,
                "nome" => $variacao->Titulo,
                "preco" => (float)$variacao->Preco,
                "tamanho" => empty($variacao->Grade_Tamanho) ? null : $variacao->Grade_Tamanho,
                "cor" => empty($variacao->Grade_Cor) ? null : $variacao->Grade_Cor,
                "tipo" => empty($variacao->Grade_Genero) ? null : $variacao->Grade_Genero,
                "especificacao" => empty($variacao->Grade_Especificacao) ? null : $variacao->Grade_Especificacao,
                "imagem" => $imagemProduto
            ];
        }
        $produtos = [];
        $buscarProdutos = sizeof($whereProdutos) > 0 ? Produto::buscarPrecos(
            new DatabaseQuery([
                (new Where("Id"))->in($whereProdutos)
            ]),
            ["Id", "Titulo", "Preco", "Codigo", "Foto", "Caminho_Foto"]
        ) : [];
        foreach($buscarProdutos as $produto) {
            $codigoProduto = preg_replace("/[^0-9]/", "", $produto->Codigo);
            $imagemProduto = "";

            if(!empty($produto->Foto)) {
                if(file_exists("../../produto/fotos/$codigoProduto/original/$produto->Foto")) {
                    $imagemProduto = "https://$produto->Caminho_Foto/produto/fotos/$codigoProduto/original/$produto->Foto";
                }
            }
            $produtos[] = (object)[
                "id" => $produto->Id,
                "nome" => $produto->Titulo,
                "preco" => (float)$produto->Preco,
                "imagem" => $imagemProduto
            ];
        }
        $produtosCarrinho = array_merge($variacoes, $produtos);

        header("Content-Type: application/json");
        echo json_encode([
            "produtos" => $produtosCarrinho,
            "metaDescontoSite" => $dadosEmpresa->Meta_Desconto_Site,
            "valorMinimoDesconto" => (float)$dadosEmpresa->Valor_Minimo_Desconto_Site,
            "percentualDesconto" => (float)$dadosEmpresa->Percentual_Desconto_Site
        ]);
    }

    public static function buscarFiltros()
    {
        $filtros = Produto::filtrosOrdenacao();
        
        foreach($filtros as $indice=> $filtro) {
            $filtros[$indice] = codificarString($filtro, "ISO-8859-1", "UTF-8");
        }
        header('Content-Type: application/json');
        echo json_encode($filtros);
    }
}