<?php
namespace LojaVirtual\App\Model;

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Database\DatabaseQuery;
use LojaVirtual\App\Config\Database\GroupBy;
use LojaVirtual\App\Config\Database\Where;
use PDO;

class Categoria
{
    public $id;
    public $nome;
    public $quantidadeProdutos;

    public function __construct(string $nome)
    {
        $this->nome = $nome;
    }

    public function setId(int $id): void
    {
        $this->id = $id;
    }

    public static function buscarTodas(?DatabaseQuery $consulta = null, ?array $colunas = null): array
    {
        if($colunas) {
            $colunas = implode(", categoria_produto.", $colunas);
            $colunas = "categoria_produto.$colunas";
        }
        $colunasConsulta = $colunas ?? "categoria_produto.Id, categoria_produto.Titulo";
        $sql = "SELECT $colunasConsulta, COUNT(produto.Id) quantidadeProdutos FROM categoria_produto
            LEFT JOIN (SELECT Id, Id_Categoria_Produto FROM produto WHERE Ativo = 'Sim' AND Tipo_Produto = 'P' AND Site = 'Sim' AND Id_Produto_Pai = '') produto
            ON produto.Id_Categoria_Produto = categoria_produto.Id";

        if($consulta) {
            $consulta = DatabaseQuery::addTableName($consulta, "categoria_produto");
            $groupBy = new GroupBy(
                $consulta->getGroupBy() ? "{$consulta->getGroupBy()->column}, categoria_produto.id" : "categoria_produto.id"
            );
            $consulta = new DatabaseQuery(
                array_merge($consulta->getWhere(), [
                    new Where('categoria_produto.Ativo', 'Sim'),
                    new Where('categoria_produto.Site', 'Sim'),
                    new Where('categoria_produto.Tipo', 'Pode vender'),
                    new Where('categoria_produto.Tipo_Categoria', 'P'),
                ]),
                $consulta->getOrder(),
                $consulta->getLimit(),
                $groupBy,
                $consulta->getHaving()
            );
            $sql .= $consulta->getFilter();
        } else {
            $sql .= " WHERE categoria_produto.Ativo = ? AND categoria_produto.Site = ? AND categoria_produto.Tipo = ? AND categoria_produto.Tipo_Categoria = ?";
        }

        $consultarCategorias = BancoDados::consultar(
            $sql,
            $consulta ? $consulta->getFilterValues() : ['Sim', 'Sim', 'Pode vender', 'P']
        );
        $obterCategorias = $consultarCategorias->fetchAll(PDO::FETCH_OBJ);
        $categorias = [];

        foreach($obterCategorias as $itemCategoria) {
            $categoria = new Categoria(
                $itemCategoria->Titulo,
            );
            $categoria->setId($itemCategoria->Id);
            $categoria->quantidadeProdutos = $itemCategoria->quantidadeProdutos;
            $categorias[] = $categoria;
        }
        return $categorias;
    }
}