<?php
namespace LojaVirtual\App\Model;

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Database\DatabaseQuery;
use LojaVirtual\App\Config\Database\OrderBy;
use LojaVirtual\App\Config\Database\Where;
use LojaVirtual\App\Model\Type\Customer\Endereco;
use PDO;

class Cliente
{
    public $id;
    public $titulo;
    public $razaoSocial;
    public $cpfCnpj;
    public $codigo;
    public $natureza;
    public $email;
    public $telefone;
    public $senha;
    public $idCfop;
    private $idVendedor;

    public function __construct(string $titulo, string $razaoSocial, string $cpfCnpj, string $natureza, string $email, string $telefone,
        string $senha, int $idCfop, ?string $idVendedor = null
    )
    {
        $this->titulo = $titulo;
        $this->razaoSocial = $razaoSocial;
        $this->cpfCnpj = $cpfCnpj;
        $this->natureza = $natureza;
        $this->email = $email;
        $this->telefone = $telefone;
        $this->senha = $senha;
        $this->idCfop = $idCfop;
        $this->idVendedor = $idVendedor;
    }

    public function salvar()
    {
        $id = md5($this->cpfCnpj.microtime());
        $getIdSuporte = BancoDados::consultar("SELECT Id FROM usuario WHERE login = ?", ["suporte"]);
        $idUsuario = $getIdSuporte->fetchObject()->Id;
        $getCodigoCliente = BancoDados::consultar("SELECT Codigo FROM cliente ORDER BY Data_Cadastro DESC LIMIT 1");
        $codigoCliente = intval($getCodigoCliente->fetchObject()->Codigo) + 1;
        $codigoCliente = str_pad("$codigoCliente", 4, "0", STR_PAD_LEFT);
        $vendedor = BancoDados::consultar("SELECT Id FROM funcionario WHERE Titulo = ?", ["Loja virtual"]);
        $this->idVendedor = $vendedor->fetchObject()->Id;
        $dataCadastro = date("Y-m-d H:i:s");

        $cadastrar = BancoDados::consultar(
            "INSERT INTO cliente SET Id = ?, Id_Usuario = ?, Id_Usuario_Upd = ?, Data_Cadastro = ?,
                Data_Cadastro_Upd = ?, Titulo = ?, Razao_Social = ?, Cpf_Cnpj = ?, Codigo = ?, Natureza = ?, Email = ?, Fone1 = ?,
                Senha_Site = ?, Id_Vendedor1 = ?, Id_Cfop = ?",
            [
                $id, $idUsuario, $idUsuario, $dataCadastro, $dataCadastro, $this->titulo, $this->razaoSocial, $this->cpfCnpj, $codigoCliente,
                $this->natureza, $this->email, $this->telefone, $this->senha, $this->idVendedor, $this->idCfop
            ], false
        );

        if($cadastrar->rowCount() > 0) {
            $this->id = $id;
        }

        return $cadastrar->rowCount() > 0;
    }

    public static function buscar(?DatabaseQuery $consulta = null, ?array $colunas = null)
    {
        if($colunas) {
            $colunas = implode(", ", $colunas);
        }
        $sql = $colunas? "SELECT $colunas FROM cliente" : "SELECT Id, Titulo, Razao_Social, Cpf_Cnpj, Codigo, Natureza, Email, Fone1,
            Senha_Site, Rua, Numero, Complemento, Cep, Bairro, Cidade, Codigo_Cidade, Estado, Id_Vendedor1, Id_Cfop FROM cliente";

        if($consulta) {
            $consulta = new DatabaseQuery(
                array_merge($consulta->getWhere(), [
                    new Where('Ativo', 'Sim'),
                    new Where('Tipo_Cliente', 'Cliente')
                ]),
                $consulta->getOrder(),
                $consulta->getLimit(),
                $consulta->getGroupBy(),
                $consulta->getHaving()
            );
            $sql .= $consulta->getFilter();
        } else {
            $sql .= " WHERE Ativo = ? AND Tipo_Cliente = ?";
        }

        $consultarCliente = BancoDados::consultar(
            $sql,
            $consulta ? $consulta->getFilterValues() : ['Sim', 'Cliente']
        );

        $obterCliente = $consultarCliente->fetchObject();
        if($obterCliente !== false) {
            $cliente = new Cliente(
                $obterCliente->Titulo,
                $obterCliente->Razao_Social,
                $obterCliente->Cpf_Cnpj,
                $obterCliente->Natureza,
                $obterCliente->Email,
                $obterCliente->Fone1,
                $obterCliente->Senha_Site,
                $obterCliente->Id_Cfop,
                $obterCliente->Id_Vendedor1
            );
            $cliente->id = $obterCliente->Id;
            $cliente->codigo = $obterCliente->Codigo;
    
            return $cliente;
        }
        return null;
    }

    public function getIdVendedor()
    {
        return $this->idVendedor;
    }

    public static function atualizarEnderecoEntrega(string $idCliente, Endereco $endereco)
    {
        $getIdCidade = BancoDados::consultar("SELECT Id FROM cidade_ibge WHERE Codigo = ?", [$endereco->ibge]);
        $idCidade = $getIdCidade->fetchObject()->Id;
        $atualizarEndereco = BancoDados::consultar(
            "UPDATE cliente SET Endereco_Entrega = ?, Cep_Entrega = ?, Bairro_Entrega = ?, Complemento_Entrega = ?, Cidade_Entrega = ?,
                Id_Cidade_Entrega = ?, Estado_Entrega = ? WHERE Id = ?",
            [
                "$endereco->logradouro, $endereco->numero", $endereco->cep, $endereco->bairro, $endereco->complemento, $endereco->cidade,
                $idCidade, $endereco->uf, $idCliente
            ],
            false
        );

        return $atualizarEndereco->rowCount() > 0;
    }

    public static function enderecos(string $idCliente, ?int $idEndereco = null) {
        $enderecos = [];
        $sqlEndereco = "SELECT * FROM endereco_cliente WHERE Id_Cliente = ?";
        $whereEndereco = [$idCliente];
        
        if($idEndereco) {
            $sqlEndereco .= " AND Id = ?";
            $whereEndereco[] = $idEndereco;
        }
        $consultarEnderecos = BancoDados::consultar($sqlEndereco, $whereEndereco);
        $infoEnderecos = $consultarEnderecos->fetchAll(PDO::FETCH_OBJ);

        foreach($infoEnderecos as $infoEndereco) {
            $endereco = new Endereco(
                $infoEndereco->Contato,
                $infoEndereco->Telefone,
                $infoEndereco->Estado,
                $infoEndereco->Cidade,
                $infoEndereco->Bairro,
                $infoEndereco->Rua,
                $infoEndereco->Numero,
                $infoEndereco->Cep,
                $infoEndereco->Ibge,
                $infoEndereco->Complemento,
                $infoEndereco->Padrao
            );
            $endereco->id = $infoEndereco->Id;
            $enderecos[] = $endereco;
        }
        return $enderecos;
    }

    public static function enderecoPadrao(string $idCliente)
    {
        [$endereco] = Endereco::buscar(
            new DatabaseQuery(
                [
                    new Where('Padrao', 'Sim'),
                    new Where('Id_Cliente', $idCliente)
                ]
            )
        );

        return $endereco;
    }

    public static function pedidos(string $idCliente, ?string $idPedido = null)
    {
        $wherePedidos = [
            new Where('Id_Cliente', $idCliente)
        ];

        if($idPedido) {
            $wherePedidos[] = new Where('Id', $idPedido);
        }

        $pedidos = Pedido::buscarTodos(
            new DatabaseQuery(
                $wherePedidos,
                [
                    new OrderBy('Data_Cadastro', 'DESC')
                ]
            )
        );

        return $pedidos;
    }
}