<?php
namespace LojaVirtual\App\Model\Pagamento;

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Empresa;
use LojaVirtual\App\Config\ErroLoja;
use LojaVirtual\App\Config\Http\ClientRequest;
use MercadoPago\Src\Config\Autenticacao;
use MercadoPago\Src\Config\Http\Api;
use MercadoPago\Src\Config\Http\Request;
use MercadoPago\Src\Entidade\IdentificacaoPagador;
use MercadoPago\Src\Entidade\Pagador;
use MercadoPago\Src\Entidade\EnderecoPagador;
use MercadoPago\Src\Pagamento;
use LojaVirtual\App\Model\Pagamento\ProcessamentoPagamento;

class PagamentoMercadoPago
{
    public $dadosPagamento;
    private $pagador;
    private $pagamento;
    private const TIPOS_PAGAMENTOS_MP = [
        "debit_card"=> "Cartao_Debito",
        "credit_card"=> "Cartao_Credito",
        "bank_transfer"=> "Pix",
        "ticket"=> "Boleto",
        "account_money"=> "Saldo_Carteira"
    ];
    private const STATUS_PAGAMENTO_MP = [
        "pending"=> "Pendente",
        "approved"=> "Aprovado",
        "authorized"=> "Autorizado",
        "in_process"=> "Processando",
        "in_mediation"=> "Disputa",
        "rejected"=> "Rejeitado",
        "cancelled"=> "Cancelado",
        "refunded"=> "Estornado",
        "charged_back"=> "Estornado"
    ];

    public function __construct(object $dadosPagamento, string $idInterno)
    {
        $this->dadosPagamento = $dadosPagamento;
        $this->pagador = new Pagador(
            addslashes($this->dadosPagamento->payer->email)
        );
        $this->pagador->first_name = $this->dadosPagamento->payer->first_name;
        $this->pagador->last_name = $this->dadosPagamento->payer->last_name;

        if(isset($this->dadosPagamento->payer->identification)) {
            $identificacaoPagador = new IdentificacaoPagador(
                addslashes($this->dadosPagamento->payer->identification->number),
                addslashes($this->dadosPagamento->payer->identification->type)
            );
            $this->pagador->setIdentificacao($identificacaoPagador);
        }

        if(isset($this->dadosPagamento->payer->address)) {
            $enderecoPagador = new EnderecoPagador(
                addslashes($this->dadosPagamento->payer->address->city),
                addslashes($this->dadosPagamento->payer->address->federal_unit),
                addslashes($this->dadosPagamento->payer->address->neighborhood),
                addslashes($this->dadosPagamento->payer->address->street_name),
                addslashes($this->dadosPagamento->payer->address->street_number),
                addslashes($this->dadosPagamento->payer->address->zip_code)
            );
            $this->pagador->setEndereco($enderecoPagador);
        }

        $this->pagamento = new Pagamento(
            (float)$this->dadosPagamento->transaction_amount,
            $this->pagador,
            $this->dadosPagamento->payment_method_id,
            $idInterno,
            $dadosPagamento->token ?? null,
            $dadosPagamento->installments ?? null,
            $dadosPagamento->description ?? null,
            $dadosPagamento->issuer_id ?? null,
            $dadosPagamento->additional_info ?? null,
            $dadosPagamento->notification_url ?? null,
        );
    }

    public function processarPagamento()
    {
        $processarPagamento = $this->pagamento->processar();
        // var_dump($processarPagamento);
        
        if(isset($processarPagamento->id)) {
            $tipoPagamentoExterno = $this::TIPOS_PAGAMENTOS_MP[$processarPagamento->payment_method->type];
            $getTipoPagamento = BancoDados::consultar("SELECT Id, Prazo01 FROM tipo_pagamento WHERE Tipo_Pagamento_Site = '$tipoPagamentoExterno' LIMIT 1");
            $tipoPagamentoInterno = $getTipoPagamento->fetchObject();
            $prazoPagamento = intval($tipoPagamentoInterno->Prazo01);
            $dataPagamentoInterno = $prazoPagamento > 0 ? date("Y-m-d", strtotime("+$prazoPagamento day")) : date("Y-m-d");

            $dadosProcessamento = new ProcessamentoPagamento(
                $processarPagamento->id,
                $tipoPagamentoInterno->Id,
                $processarPagamento->transaction_amount,
                $this::STATUS_PAGAMENTO_MP[$processarPagamento->status],
                $processarPagamento->date_created,
                $processarPagamento->date_approved ?? '',
                $dataPagamentoInterno,
                $processarPagamento->taxes_amount,
                $processarPagamento->installments ?? 0,
                $processarPagamento->transaction_details->installment_amount ?? $processarPagamento->transaction_amount,
                $processarPagamento->payment_type_id,
                $processarPagamento->transaction_data ?? null
            );
            return $dadosProcessamento;
        }
        return null;
    }

    public static function detalhes(int $id)
    {
        $tokenAcesso = Autenticacao::getTokenAcesso();
        $url = Api::getUrl()."/".Api::getVersao()."/payments/$id";
        $cabecalho = [
            "Authorization" => "Bearer $tokenAcesso"
        ];
        $buscarDetalhes = ClientRequest::get($url, $cabecalho);

        return $buscarDetalhes->body(); 
    }

    public static function getStatusInterno(string $statusExterno) {
        return self::STATUS_PAGAMENTO_MP[$statusExterno];
    }

    public static function atualizarStatusInterno($detalhesPagamento)
    {
        $dadosLoja = Empresa::dados();
        $statusInterno = self::getStatusInterno($detalhesPagamento->status);
        $idInternoPagamento = addslashes($detalhesPagamento->external_reference);

        if(strlen($statusInterno) > 0) {
            $tabelaPagamento = $dadosLoja->Tabela_Pedidos_Site == "Venda" ? "venda_tipo_pagamento" : "pdv_tipo_pagamento";
            $atualizarStatus = BancoDados::consultar("UPDATE $tabelaPagamento SET Status_Site = '$statusInterno' WHERE Id = '$idInternoPagamento'");

            $statusCancelamento = [
                'Rejeitado',
                'Cancelado',
                'Estornado'
            ];
            if(in_array($statusInterno, $statusCancelamento)) {
                switch($dadosLoja->Tabela_Pedidos_Site) {
                    case "Venda":
                        $getIdVenda = BancoDados::consultar("SELECT Id_Venda FROM venda_tipo_pagamento WHERE Id = '$idInternoPagamento'");
                        $idVenda = $getIdVenda->fetchObject()->Id_Venda;
                        $atualizarStatus = BancoDados::consultar("UPDATE venda SET Status = 'Cancelado' WHERE Id = '$idVenda'");
                        BancoDados::consultar("UPDATE estoque SET Concluido = 'Cancelado' WHERE Id_Externo = '$idVenda'");
                        break;
                    case "PDV":
                        $getIdPdv = BancoDados::consultar("SELECT Id_Pdv FROM pdv_tipo_pagamento WHERE Id = '$idInternoPagamento'");
                        $idPdv = $getIdPdv->fetchObject()->Id_Pdv;
                        $atualizarStatus = BancoDados::consultar("UPDATE pdv SET Status = 'Cancelado' WHERE Id = '$idPdv'");
                        BancoDados::consultar("UPDATE estoque SET Concluido = 'Cancelado' WHERE Id_Externo = '$idPdv'");
                        break;
                }
            }
            return $atualizarStatus->rowCount() > 0;
        }
        return false;
    }
}