<?php
namespace LojaVirtual\App\Model;
include "app/model/pedido.php";

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Empresa;
use LojaVirtual\App\Model\Pagamento\ProcessamentoPagamento;
use LojaVirtual\App\Model\Pedido;
use LojaVirtual\App\Model\Type\Order\Entrega;
use LojaVirtual\App\Model\Pagamento;

class PDV extends Pedido
{
    public $dataPdv;
    public $tipoDesconto;
    public $tipoPedido;

    /**
     * 
     * @param int|Pagamento\PagamentoMercadoPago $pagamento
     */
    public function __construct(float $valor, float $valorPago, string $status, array $produtos, int $idUnidade, string $idVendedor,
        string $idFuncionario, string $idCliente, $pagamento, string $observacao, float $desconto, Entrega $entrega,
        float $valorFrete, int $idCfop, string $tipoDesconto, string $dataCadastro
    )
    {
        $this->id = md5($idCliente.microtime());
        $this->valor = $valor;
        $this->valorPago = $valorPago;
        $this->status = $status;
        $this->produtos = $produtos;
        $this->idUnidade = $idUnidade;
        $this->idVendedor = $idVendedor;
        $this->idFuncionario = $idFuncionario;
        $this->idCliente = $idCliente;
        $this->pagamento = $pagamento;
        $this->observacao = $observacao;
        $this->desconto = $desconto;
        $this->entrega = $entrega;
        $this->valorFrete = $valorFrete;
        $this->idCfop = $idCfop;
        $this->tipoDesconto = $tipoDesconto;
        $this->dataPdv = date("Y-m-d");
        $this->dataCadastro = $dataCadastro;
    }

    public function salvar()
    {
        $this->desconto = $this->desconto > 0 && $this->tipoDesconto == "Perc" ? ($this->valor * ($this->desconto / 100)) : $this->desconto;
        $this->tipoDesconto = $this->tipoDesconto == "Perc" ? "Real" : $this->desconto;
        $idTipoPagamento = is_int($this->pagamento) ? $this->pagamento : null; 

        if(!$idTipoPagamento) {
            $pagamento = $this->processarPagamento();
            $idTipoPagamento = $pagamento->idInternoTipoPagamento;

            $statusRejeicaoPagamento = [
                "Disputa",
                "Rejeitado",
                "Cancelado",
                "Estornado"
            ];

            if(!$pagamento || in_array($pagamento->status, $statusRejeicaoPagamento)) {
                return false;
            }
        }

        $cadastrar = BancoDados::consultar(
            "INSERT INTO pdv SET Id = ?, Data_Cadastro = ?, Data_Cadastro_Upd = ?, Data_Pdv = ?, Id_Funcionario = ?, Id_Cliente = ?,
                Id_Tipo_Pagamento = ?, Total_Pagar = ?, Valor_Pago = ?, Desconto_Total = ?, Tipo_Desconto = ?, Id_Endereco_Entrega = ?, Status = ?,
                Obs = ?, Id_Unidade = ?, Id_Vendedor = ?, Id_Cfop = ?, Entrega = ?, Tipo_Pedido = ?, Valor_Taxa_Entrega = ?, Cobrar_Taxa_Entrega = ?",
            [
                $this->id, $this->dataCadastro, $this->dataCadastro, $this->dataPdv, $this->idFuncionario, $this->idCliente, $idTipoPagamento,
                $this->valor, $this->valorPago, $this->desconto, $this->tipoDesconto, $this->entrega->idEndereco, $this->status, $this->observacao,
                $this->idUnidade, $this->idVendedor, $this->idCfop, "Sim", "Online", $this->valorFrete, "Sim"
            ],
            false
        );

        if($cadastrar->rowCount() > 0) {
            $this->salvarProdutos();
            if(isset($pagamento)) {
                $this->salvarPagamento($pagamento);
            } else {
                $this->salvarPagamento();
            }
        }

        return $cadastrar->rowCount() > 0 ? $this->id : false;
    }

    private function salvarProdutos()
    {
        $getCliente = BancoDados::consultar(
            "SELECT Estado, Estado_Entrega, Id_Regime_Especial FROM cliente WHERE Id = ?", 
            [$this->idCliente]
        );
        $cliente = $getCliente->fetchObject();
        $empresa = Empresa::dados();
        $desconto = 0;

        foreach ($this->produtos as $produto) {
            $idPdvProduto = md5($produto->id.microtime());
            $getProduto = BancoDados::consultar(
                "SELECT Titulo, Codigo, Id_Categoria_Produto, Id_Categoria_Produto_Filho, Custo, Gtin, Id_Cfop_Venda, Id_Cfop_Venda_F,
                    Codigo_ST, Codigo_ST2, CST_Ipi, Aliquota_Ipi, CST_Pis, Aliquota_Pis, CST_Cofins, Aliquota_Cofins, Id_Local_Estoque_Saida
                    FROM produto WHERE Id = ? AND Ativo = ? AND Tipo_Produto = ? AND Site = ?",
                [$produto->id, "Sim", "P", "Sim"]
            );
            
            if($getProduto->rowCount() > 0) {
                $impostosProduto = Produto::impostos(
                    $produto->id,
                    $cliente->Id_Regime_Especial,
                    $empresa->Id,
                    $empresa->Estado,
                    $cliente->Estado
                );
                $dadosProduto = $getProduto->fetchObject();
                $idCfopProduto = $cliente->Estado == $empresa->Estado ? $dadosProduto->Id_Cfop_Venda : $dadosProduto->Id_Cfop_Venda_F;
                $idCfopProduto = !empty($idCfopProduto) ? (int)$idCfopProduto : $this->idCfop;
                $codigoST = $cliente->Estado == $empresa->Estado ? $dadosProduto->Codigo_ST : $dadosProduto->Codigo_ST2;
                $codigoST = $impostosProduto->Codigo_ST ?? $codigoST;
                $aliquotaIpi = $impostosProduto->Aliquota_Ipi ?? $dadosProduto->Aliquota_Ipi;
                $cstIpi = $impostosProduto->CST_Ipi ?? $dadosProduto->CST_Ipi;
                $aliquotaPis = $impostosProduto->Aliquota_Pis ?? $dadosProduto->Aliquota_Pis;
                $cstPis = $impostosProduto->CST_Pis ?? $dadosProduto->CST_Pis;
                $aliquotaCofins = $impostosProduto->Aliquota_Cofins ?? $dadosProduto->Aliquota_Cofins;
                $cstCofins = $impostosProduto->CST_Cofins ?? $dadosProduto->CST_Cofins;
                $icms = $impostosProduto->Aliquota_Icms ?? 0.00;
                $icmsSub = $impostosProduto->Icms_Sub ?? 0.00;
                $mva = $impostosProduto->Mva ?? 0.00;
                $reducaoIcms = $impostosProduto->Reducao_Icms ?? 0.00;
                $descontoReal = $this->desconto > 0 ? $this->calcularDescontoProduto($produto->preco, $produto->quantidade, $this->desconto) : 0;

                $cadastrar = BancoDados::consultar(
                    "INSERT INTO pdv_produto SET Id = ?, Data_Cadastro = ?, Id_Pdv = ?, Id_Produto = ?, Id_Funcionario = ?, Id_Cliente = ?,
                        Preco = ?, Qtd = ?, Desconto = ?, Desconto_Real_Pe_Nota = ?, Preco_Custo = ?, Ipi = ?, CST_Ipi = ?, Aliquota_Pis = ?, CST_Pis = ?,
                        Aliquota_Cofins = ?, CST_Cofins = ?, Icms = ?, Icms_Sub = ?, Mva = ?, Reducao = ?, Cancelado = ?, Id_Categoria_Produto = ?,
                        Id_Categoria_Produto_Filho = ?, Id_Cfop = ?, ST_Icms = ?, Gtin = ?, Id_Local_Estoque = ?",
                    [
                        $idPdvProduto, $this->dataCadastro, $this->id, $produto->id, $this->idFuncionario, $this->idCliente, $produto->preco,
                        $produto->quantidade, $desconto, $descontoReal, $dadosProduto->Custo, $aliquotaIpi, $cstIpi, $aliquotaPis, $cstPis,
                        $aliquotaCofins, $cstCofins, $icms, $icmsSub, $mva, $reducaoIcms, "No", $dadosProduto->Id_Categoria_Produto,
                        $dadosProduto->Id_Categoria_Produto_Filho, $idCfopProduto, $codigoST, $dadosProduto->Gtin, $dadosProduto->Id_Local_Estoque_Saida
                    ],
                    false
                );

                if($cadastrar->rowCount() > 0) {
                    Produto::baixarEstoque(
                        $idPdvProduto,
                        $produto->id,
                        (float)$produto->quantidade,
                        (float)$dadosProduto->Id_Local_Estoque_Saida,
                        (float)$dadosProduto->Custo,
                        (float)$produto->preco,
                        $idCfopProduto,
                        (float)$icms,
                        (float)$aliquotaIpi,
                        "PDV",
                        "No",
                        $this->id,
                        $this->idUnidade,
                        (float)$icmsSub,
                        (float)$mva,
                        (float)$reducaoIcms,
                        $this->idCliente
                    );
                }
            }
        }
    }

    private function salvarPagamento(?ProcessamentoPagamento $processamentoPagamento = null)
    {
        $idPagamento = $this->idPagamento ?? md5($this->id.microtime());
        $idTipoPagamento = !$processamentoPagamento ? $this->pagamento : $processamentoPagamento->idInternoTipoPagamento;
        $totalPagar = !$processamentoPagamento ? $this->valor : $processamentoPagamento->valorPago;
        $valorPago = !$processamentoPagamento ? 0 : $processamentoPagamento->valorPago;
        $statusPagamento = $processamentoPagamento->status ?? "";

        $cadastrar = BancoDados::consultar(
            "INSERT INTO pdv_tipo_pagamento SET Id = ?, Data_Cadastro = ?, Id_Pdv = ?, Id_Tipo_Pagamento = ?, Id_Funcionario = ?, Id_Cliente = ?,
                Total_Pagar = ?, Valor_Pago = ?, Ordem = ?, Status_Site = ?, Id_Plataforma = ?",
            [
                $idPagamento, $this->dataCadastro, $this->id, $idTipoPagamento, $this->idFuncionario, $this->idCliente, $totalPagar, $valorPago, 1,
                $statusPagamento, $processamentoPagamento->id
            ],
            false
        );
        return $cadastrar->rowCount() > 0;
    }
}