<?php
namespace LojaVirtual\App\Model;

use Loja_virtual\App\Model\Pagamento\PagamentoRealizado;
use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Database\DatabaseQuery;
use LojaVirtual\App\Config\Database\Where;
use LojaVirtual\App\Config\Empresa;
use LojaVirtual\App\Config\ErroLoja;
use LojaVirtual\App\Model\PreVenda;
use LojaVirtual\App\Model\Type\Customer\Endereco;
use LojaVirtual\App\Model\Type\Order\ProdutoPedido;
use LojaVirtual\App\Model\Venda;
use LojaVirtual\App\Model\Pagamento\PagamentoMercadoPago;
use PDO;

class Pedido
{
    public $id;
    public $dataCadastro;
    public $idFuncionario;
    public $idCliente;
    public $pagamento;
    public $status;
    public $valor;
    public $valorPago;
    public $lancamento;
    public $idUnidade;
    public $idCfop;
    public $idVendedor;
    public $site;
    private $objeto;
    public $produtos;
    public $desconto;
    public $valorFrete;
    public $entrega;
    public $observacao;
    public $transacao;
    public $pagamentosRealizados;
    protected $idPagamento;

    /**
     * 
     * @param int|object $pagamento
     */
    public function __construct(float $valor, float $valorPago, string $status, array $produtos, float $desconto, $entrega, int $idUnidade,
        string $idVendedor, string $idFuncionario, string $idCliente, $pagamento, string $observacao, int $idCfop, float $valorFrete,
        string $dataCadastro
    )
    {
        $this->valor = $valor;
        $this->valorPago = $valorPago;
        $this->status = $status;
        $this->produtos = $produtos;
        $this->desconto = $desconto;
        $this->entrega = $entrega;
        $this->idUnidade = $idUnidade;
        $this->idVendedor = $idVendedor;
        $this->idFuncionario = $idFuncionario;
        $this->idCliente = $idCliente;
        $this->pagamento = $pagamento;
        $this->observacao = $observacao;
        $this->idCfop = $idCfop;
        $this->valorFrete = $valorFrete;
        $this->dataCadastro = $dataCadastro;
    }

    private function criarPedido()
    {
        $dadosLoja = Empresa::dados();
        $tabelaPedido = $dadosLoja->Tabela_Pedidos_Site;

        switch ($tabelaPedido) {
            case "Pr-Venda":
                $this->criarPreVenda();
                break;
            case "Venda":
                $this->criarVenda();
                break;
            case "PDV":
                $this->criarPdv();
                break;
        }
    }
    private function criarPreVenda()
    {
        $descontoReal = 0;

        $preVenda = new PreVenda(
            $this->valor,
            $this->valorPago,
            "Aberto",
            $this->produtos,
            $this->idUnidade,
            $this->idVendedor,
            $this->idFuncionario,
            $this->idCliente,
            $this->pagamento,
            $this->observacao,
            $this->entrega,
            $this->desconto,
            $descontoReal,
            $this->valorFrete,
            $this->idCfop,
            $this->dataCadastro
        );
        $this->objeto = $preVenda;
    }

    private function criarVenda()
    {
        $descontoReal = 0;

        $venda = new Venda(
            $this->valor,
            $this->valorPago,
            "Aberto",
            $this->produtos,
            $this->idUnidade,
            $this->idVendedor,
            $this->idFuncionario,
            $this->idCliente,
            $this->pagamento,
            $this->observacao,
            $this->entrega,
            $this->desconto,
            $descontoReal,
            $this->valorFrete,
            $this->idCfop,
            $this->dataCadastro
        );
        $this->objeto = $venda;
    }

    private function criarPdv()
    {
        $tipoDesconto = "Perc";
        $pdv = new PDV(
            $this->valor,
            $this->valorPago,
            $this->status,
            $this->produtos,
            $this->idUnidade,
            $this->idVendedor,
            $this->idFuncionario,
            $this->idCliente,
            $this->pagamento,
            $this->observacao,
            $this->desconto,
            $this->entrega,
            $this->valorFrete,
            $this->idCfop,
            $tipoDesconto,
            $this->dataCadastro
        );
        $this->objeto = $pdv;
    }
    
    public function getId()
    {
        return $this->id;
    }

    public function salvar()
    {
        $this->criarPedido();

        return $this->objeto->salvar();
    }

    public static function buscarTodos(DatabaseQuery $consulta)
    {
        try {
            $dadosLoja = Empresa::dados();
            $tabelaPedido = $dadosLoja->Tabela_Pedidos_Site;
            $statusPedidos = [
                "Aberto"=> "Processando",
            ];
            $sql = "SELECT Id, Lancamento, Data_Cadastro, Id_Funcionario, Id_Cliente, Id_Tipo_Pagamento, Valor_Pago, Id_Endereco_Entrega,
                Status, Obs, Id_Unidade, Id_Vendedor, Id_Cfop,";

            switch ($tabelaPedido) {
                case "Pr-Venda":
                    $sql .= "Valor_Original AS Valor, Desconto AS Desconto_Total, Valor_Frete, Site FROM pre_venda";
                    break;
                case "Venda":
                    $sql .= "Valor_Original AS Valor, Desconto AS Desconto_Total, Valor_Frete, Site FROM venda";
                    break;
                case "PDV":
                    $sql .= "Data_Pdv, Total_Pagar AS Valor, Desconto_Total, Tipo_Desconto, Tipo_Pedido, Entrega,
                        Valor_Taxa_Entrega AS Valor_Frete, Cobrar_Taxa_Entrega FROM pdv";
                    break;
            }

            $sql .= $consulta->getFilter();
            $buscarPedidos = BancoDados::consultar($sql, $consulta->getFilterValues());
            $getPedidos = $buscarPedidos->fetchAll(PDO::FETCH_OBJ);
            $pedidos = [];

            foreach($getPedidos as $dadosPedido) {
                if($tabelaPedido == "Pr-Venda" || $tabelaPedido == "Venda") {
                    $dadosPedido->Desconto_Total = $dadosPedido->Valor * ($dadosPedido->Desconto_Total / 100);
                }
                $produtosPedidos = self::buscarProdutosPedido($dadosPedido->Id);
                $dadosPedido->Cobrar_Taxa_Entrega = $dadosPedido->Cobrar_Taxa_Entrega ?? "Sim";
                $enderecoEntregaPedido = Endereco::buscar(
                    new DatabaseQuery(
                        [
                            new Where("Id", $dadosPedido->Id_Endereco_Entrega)
                        ]
                    )
                );
                $pedido = new Pedido(
                    $dadosPedido->Valor,
                    $dadosPedido->Valor_Pago,
                    isset($statusPedidos[$dadosPedido->Status]) ? $statusPedidos[$dadosPedido->Status] : $dadosPedido->Status,
                    $produtosPedidos,
                    $dadosPedido->Desconto_Total,
                    $enderecoEntregaPedido,
                    $dadosPedido->Id_Unidade,
                    $dadosPedido->Id_Vendedor,
                    $dadosPedido->Id_Funcionario,
                    $dadosPedido->Id_Cliente,
                    $dadosPedido->Id_Tipo_Pagamento,
                    $dadosPedido->Obs,
                    $dadosPedido->Id_Cfop,
                    $dadosPedido->Valor_Frete,
                    $dadosPedido->Data_Cadastro
                );
                $pedido->id = $dadosPedido->Id;
                $pedido->site = isset($dadosPedido->Site) ? $dadosPedido->Site == "Sim" : $dadosPedido->Tipo_Pedido == "Online";
                $pedido->lancamento = $dadosPedido->Lancamento;
                $pedido->pagamentosRealizados = $pedido->buscarPagamentosRealizados();
                $pedidos[] = $pedido;
            }

            return $pedidos;
        } catch (\Throwable $th) {
            new ErroLoja($th);
        }
    }

    public static function buscarProdutosPedido(string $idPedido)
    {
        $dadosLoja = Empresa::dados();
        $tabelaPedido = $dadosLoja->Tabela_Pedidos_Site;
        $tabelaProdutosPedido = $tabelaPedido == "Pr-Venda" ? "pre_venda_produto" : ($tabelaPedido == "Venda" ? "venda_produto" : "pdv_produto");
        $colunaWhere = $tabelaProdutosPedido == "pre_venda_produto" ? "Id_Pre_Venda" : ($tabelaPedido == "Venda" ? "Id_Venda" : "Id_Pdv");
        $sql = "SELECT produto.Titulo, produto_pai.Titulo Titulo_Pai, produto.Codigo, produto.Caminho_Foto, produto.Foto, produto.Grade_Cor,
            produto.Grade_Tamanho, produto.Grade_Genero, produto.Grade_Especificacao, $tabelaProdutosPedido.Id, $tabelaProdutosPedido.Data_Cadastro,
            $tabelaProdutosPedido.Id_Produto, $tabelaProdutosPedido.Preco, $tabelaProdutosPedido.Qtd";
        $sql .= $tabelaProdutosPedido != "pre_venda_produto" ? ", $tabelaProdutosPedido.Cancelado" : ""; 
        $sql .= " FROM $tabelaProdutosPedido INNER JOIN produto ON produto.Id = $tabelaProdutosPedido.Id_Produto
            LEFT JOIN produto produto_pai ON produto_pai.Id = produto.Id_Produto_Pai
            WHERE $tabelaProdutosPedido.$colunaWhere = ?";
        $produtosPedido = [];

        $buscarProdutos = BancoDados::consultar($sql, [$idPedido]);
        $getProdutos = $buscarProdutos->fetchAll(PDO::FETCH_OBJ);
    
        foreach($getProdutos as $dadosProduto) {
            $imagemProduto = "";
            $codigoProduto = preg_replace("/[^0-9]/", "", $dadosProduto->Codigo);
            $produtoCancelado = $dadosProduto->Cancelado ?? codificarString("No", "ISO-8859-1", "UTF-8");

            if(!empty($dadosProduto->Foto)) {
                if(file_exists("../../produto/fotos/$codigoProduto/original/$dadosProduto->Foto")) {
                    $imagemProduto = "https://$dadosProduto->Caminho_Foto/produto/fotos/$codigoProduto/original/$dadosProduto->Foto";
                }
            }

            $produtosPedido[] = new ProdutoPedido(
                $dadosProduto->Id,
                strlen($dadosProduto->Titulo_Pai) > 0 ? $dadosProduto->Titulo_Pai : $dadosProduto->Titulo,
                $dadosProduto->Data_Cadastro,
                $dadosProduto->Id_Produto,
                $dadosProduto->Preco,
                $dadosProduto->Qtd,
                $produtoCancelado,
                $imagemProduto,
                $dadosProduto->Grade_Tamanho,
                $dadosProduto->Grade_Cor,
                $dadosProduto->Grade_Genero,
                $dadosProduto->Grade_Especificacao
            );
        }
        return $produtosPedido;
    }

    public function calcularDescontoProduto(float $preco, float $quantidade, float $percentualDesconto)
    {
        $descontoProduto = ($preco * $quantidade) * ($percentualDesconto / 100);
        return $descontoProduto;
    }

    public static function calcularValorEntrega(int $idEndereco)
    {
        [$endereco] = Endereco::buscar(
            new DatabaseQuery(
                [
                    new Where("Id", $idEndereco)
                ]
            )
        );
        $getValorEntrega = BancoDados::consultar("SELECT Frete FROM bairro WHERE Titulo = ?", [$endereco->bairro]);
        return $getValorEntrega->rowCount() > 0 ? $getValorEntrega->fetchObject()->Frete : 0;
    }

    protected function processarPagamento()
    {
        $dadosLoja = Empresa::dados();
        $processamentoPagamento = null;
        $this->idPagamento = md5($this->id.microtime());

        switch($dadosLoja->Plataforma_Pagamento_Site) {
            case "Mercado_Pago":
                $pagamento = new PagamentoMercadoPago($this->pagamento, $this->idPagamento);
                $processamentoPagamento = $pagamento->processarPagamento();
                $this->transacao = $processamentoPagamento->transaction_data ?? null;
                break;
        }
        return $processamentoPagamento;
    }

    private function buscarPagamentosRealizados() {
        $dadosLoja = Empresa::dados();
        $pagamentosRealizados = [];

        switch($dadosLoja->Tabela_Pedidos_Site) {
            case "Pr-Venda":
                // $getDadosPedido = BancoDados::consultar("SELECT Id_Tipo_Pagamento, Valor_Pago FROM pre_Venda WHERE Id = '$this->id'");
                // $dadosPedido = $getDadosPedido->fetchObject();
                // $getTipoPagamento = BancoDados::consultar("SELECT Tipo_Pagamento_Site")
                break;
            case "Venda":
                // $pagamentosRealizados = self::buscarPagamentosRealizadosVenda();
                break;
            case "PDV":
                $getDadosPedido = BancoDados::consultar("SELECT Id_Tipo_Pagamento, Valor_Pago, Status_Site FROM pdv_tipo_pagamento WHERE Id_Pdv = '$this->id'");

                if($getDadosPedido->rowCount() > 0) {
                    $dadosPedido = $getDadosPedido->fetchObject();

                    if($dadosPedido->Status_Site) {
                        $getDadosPagamento = BancoDados::consultar("SELECT Tipo_Pagamento_Site FROM tipo_pagamento WHERE Id = '$dadosPedido->Id_Tipo_Pagamento'");
                        $dadosPagamento = $getDadosPagamento->fetchObject();
                        $pagamentosRealizados[] = new PagamentoRealizado(
                            $dadosPagamento->Tipo_Pagamento_Site,
                            $dadosPedido->Valor_Pago,
                            $dadosPedido->Status_Site
                        );
                    }
                }
                break;
        }

        return $pagamentosRealizados;
    } 
}