<?php
namespace LojaVirtual\App\Model;

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Empresa;
use LojaVirtual\App\Model\Pedido;
use LojaVirtual\App\Model\Type\Order\Entrega;

class PreVenda extends Pedido
{
    public $dataPrevenda;
    public $descontoReal;
    public $valorFrete;

    public function __construct(float $valor, float $valorPago, string $status, array $produtos, int $idUnidade, string $idVendedor,
        string $idFuncionario, string $idCliente, int $pagamento, string $observacao, Entrega $entrega, float $desconto,
        float $descontoReal, float $valorFrete, int $idCfop, string $dataCadastro
    )
    {
        $this->id = md5($idCliente.microtime());
        $this->valor = $valor;
        $this->valorPago = $valorPago;
        $this->status = $status;
        $this->produtos = $produtos;
        $this->entrega = $entrega;
        $this->idUnidade = $idUnidade;
        $this->idVendedor = $idVendedor;
        $this->idFuncionario = $idFuncionario;
        $this->idCliente = $idCliente;
        $this->pagamento = $pagamento;
        $this->desconto = $desconto;
        $this->descontoReal = $descontoReal;
        $this->valorFrete = $valorFrete;
        $this->idCfop = $idCfop;
        $this->dataPrevenda = date('Y-m-d');
        $this->observacao = $observacao;
        $this->dataCadastro = $dataCadastro;
    }

    public function salvar()
    {
        $cadastrar = BancoDados::consultar(
            "INSERT INTO pre_venda SET Id = ?, Id_Vendedor = ?,  Id_Funcionario = ?, Data_Cadastro = ?, Data_Cadastro_Upd = ?,
                Id_Cliente = ?, Id_Tipo_Pagamento = ?, Status = ?, Valor_Original = ?, Valor_Pago = ?, Id_Unidade = ?, Id_Cfop = ?,
                Data_Pre_Venda = ?, Desconto = ?, Desconto_Real = ?, Site = ?, Valor_Frete = ?, Obs = ?, Id_Endereco_Entrega = ?",
            [
                $this->id, $this->idVendedor, $this->idVendedor, $this->dataCadastro, $this->dataCadastro, $this->idCliente,
                $this->pagamento, $this->status, $this->valor, $this->valorPago, $this->idUnidade, $this->idCfop, $this->dataPrevenda,
                $this->desconto, $this->descontoReal, 'Sim', $this->valorFrete, $this->observacao, $this->entrega->idEndereco
            ]
        );
        $pedidoCadastrado = $cadastrar->rowCount() > 0;

        if($pedidoCadastrado) {
            BancoDados::consultar(
                "UPDATE pre_venda SET Numero_Pedido = ? WHERE Id = ?", 
                [
                    BancoDados::idInserido(), $this->id
                ]
            );
            $this->salvarProdutos();
        }

        return $pedidoCadastrado ? $this->id : false;
    }

    private function salvarProdutos()
    {
        $getCliente = BancoDados::consultar(
            "SELECT Estado, Estado_Entrega, Id_Regime_Especial FROM cliente WHERE Id = ?", 
            [$this->idCliente]
        );
        $cliente = $getCliente->fetchObject();
        $empresa = Empresa::dados();

        foreach($this->produtos as $produto) {
            $idPreVendaProduto = md5($produto->id.microtime());
            $getProduto = BancoDados::consultar(
                "SELECT Titulo, Codigo, Id_Categoria_Produto, Id_Categoria_Produto_Filho, Custo, Gtin, Cest, Id_Cfop_Venda, Id_Cfop_Venda_F,
                    Codigo_ST, Codigo_ST2, CST_Ipi, Aliquota_Ipi, CST_Pis, Aliquota_Pis, CST_Cofins, Aliquota_Cofins, Id_Local_Estoque_Saida
                    FROM produto WHERE Id = ? AND Ativo = ? AND Tipo_Produto = ? AND Site = ?",
                [$produto->id, "Sim", "P", "Sim"]
            );

            if($getProduto->rowCount() > 0) {
                $impostosProduto = Produto::impostos(
                    $produto->id,
                    $cliente->Id_Regime_Especial,
                    $empresa->Id,
                    $empresa->Estado,
                    $cliente->Estado
                );
                $dadosProduto = $getProduto->fetchObject();
                $idCfopProduto = $cliente->Estado == $empresa->Estado ? $dadosProduto->Id_Cfop_Venda : $dadosProduto->Id_Cfop_Venda_F;
                $idCfopProduto = !empty($idCfopProduto) ? (int)$idCfopProduto : $this->idCfop;
                $codigoST = $cliente->Estado == $empresa->Estado ? $dadosProduto->Codigo_ST : $dadosProduto->Codigo_ST2;
                $codigoST = $impostosProduto->Codigo_ST ?? $codigoST;
                $aliquotaIpi = $impostosProduto->Aliquota_Ipi ?? $dadosProduto->Aliquota_Ipi;
                $cstIpi = $impostosProduto->CST_Ipi ?? $dadosProduto->CST_Ipi;
                $aliquotaPis = $impostosProduto->Aliquota_Pis ?? $dadosProduto->Aliquota_Pis;
                $cstPis = $impostosProduto->CST_Pis ?? $dadosProduto->CST_Pis;
                $aliquotaCofins = $impostosProduto->Aliquota_Cofins ?? $dadosProduto->Aliquota_Cofins;
                $cstCofins = $impostosProduto->CST_Cofins ?? $dadosProduto->CST_Cofins;
                $icms = $impostosProduto->Aliquota_Icms ?? 0.00;
                $icmsSub = $impostosProduto->Icms_Sub ?? 0.00;
                $mva = $impostosProduto->Mva ?? 0.00;
                $reducaoIcms = $impostosProduto->Reducao_Icms ?? 0.00;
                $reducaoBC = $impostosProduto->Reducao_BC ?? 0.00;
                $descontoRealProduto = 0;

                $cadastrar = BancoDados::consultar(
                    "INSERT INTO pre_venda_produto SET Id = ?, Id_Pre_Venda = ?, Id_Produto = ?, Preco = ?, Preco_Custo = ?, Desconto = ?,
                        Desconto_Real = ?, Qtd = ?, Liberado = ?, Id_Cliente = ?, Id_Unidade = ?, Id_Categoria_Produto = ?, Id_Cfop = ?, Id_Local_Estoque = ?,
                        Ipi = ?, CST_Ipi = ?, Aliquota_Pis = ?, CST_Pis = ?, Aliquota_Cofins = ?, CST_Cofins = ?, Icms = ?, Icms_Sub = ?, Mva = ?,
                        Reducao = ?, ReducaoBC = ?, ST_Icms = ?, Gtin = ?, Cest = ?, Data_Cadastro = ?",
                    [
                        $idPreVendaProduto, $this->id, $produto->id, $produto->preco, $dadosProduto->Custo, $this->desconto, $descontoRealProduto,
                        $produto->quantidade, $produto->quantidade, $this->idCliente, $this->idUnidade, $dadosProduto->Id_Categoria_Produto,
                        $idCfopProduto, $dadosProduto->Id_Local_Estoque_Saida, $aliquotaIpi, $cstIpi, $aliquotaPis, $cstPis, $aliquotaCofins,
                        $cstCofins, $icms, $icmsSub, $mva, $reducaoIcms, $reducaoBC, $codigoST, $dadosProduto->Gtin, $dadosProduto->Cest, $this->dataCadastro
                    ]
                );

                if($cadastrar->rowCount() > 0) {
                    Produto::baixarEstoque(
                        $idPreVendaProduto,
                        $produto->id,
                        (float)$produto->quantidade,
                        (float)$dadosProduto->Id_Local_Estoque_Saida,
                        (float)$dadosProduto->Custo,
                        (float)$produto->preco,
                        $idCfopProduto,
                        (float)$icms,
                        (float)$aliquotaIpi,
                        "PDV",
                        "No",
                        $this->id,
                        $this->idUnidade,
                        (float)$icmsSub,
                        (float)$mva,
                        (float)$reducaoIcms,
                        $this->idCliente
                    );
                }
            }
        }
    }
}