<?php
namespace LojaVirtual\App\Model;

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Database\DatabaseQuery;
use LojaVirtual\App\Config\Database\Having;
use LojaVirtual\App\Config\Database\OrderBy;
use LojaVirtual\App\Config\Database\Where;
use LojaVirtual\App\Config\Empresa;
// use LojaVirtual\App\Model\VariacaoProduto;
use PDO;

class Produto
{
    public $id;
    public $nome;
    public $preco;
    public $descricao;
    public $estoque;
    public $cores;
    public $tamanhos;
    public $tipos;
    public $especificacoes;
    public $imagens;

    public function __construct(string $id, string $nome, float $preco, string $descricao, float $estoque, array $cores, array $tamanhos, array $tipos,
        array $especificacoes, array $imagens = []
    )
    {
        $this->id = $id;
        $this->nome = $nome;
        $this->preco = $preco;
        $this->descricao = $descricao;
        $this->estoque = $estoque;
        $this->cores = $cores;
        $this->tamanhos = $tamanhos;
        $this->tipos = $tipos;
        $this->especificacoes = $especificacoes;
        $this->imagens = $imagens;
    }

    public static function buscarTodos(?DatabaseQuery $consulta = null, ?array $colunas = null): array
    {
        if($colunas) {
            $colunas = implode(", ", $colunas);
            $tabelaPreco = Empresa::tabelaPreco();
            $colunas = str_replace("Preco", "Preco$tabelaPreco Preco", $colunas);
        }
        $tabelaPreco = Empresa::tabelaPreco();
        $sqlEstoque = "SELECT SUM(Qtd) FROM estoque WHERE Id_Produto = produto.Id AND Concluido = 'Sim'";
        $colunasFotosProduto = [
            'produto.Foto',
            'produto.Foto2',
            'produto.Foto3',
            'produto.Foto4'
        ];
        $colunasFotos = implode(",", $colunasFotosProduto);
        $sql = $colunas ? "SELECT $colunas FROM produto" : "SELECT produto.Id, produto.Titulo, produto.Codigo, produto.Preco$tabelaPreco Preco,
            produto.Caracteristica, ($sqlEstoque) Qtd_Estoque, produto.Caminho_Foto, $colunasFotos, produto.Grade_Cor, produto.Grade_Tamanho,
            produto.Grade_Genero, produto.Genero_Produto, produto.Grade_Especificacao FROM produto";
        $sql .= " INNER JOIN categoria_produto ON produto.Id_Categoria_Produto = categoria_produto.Id";

        if($consulta) {
            $where = array_merge($consulta->getWhere(), [
                new Where('produto.Ativo', 'Sim'),
                new Where('produto.Tipo_Produto', 'P'),
                new Where('produto.Site', 'Sim'),
                new Where('categoria_produto.Ativo', 'Sim'),
                new Where('categoria_produto.Tipo_Categoria', 'P'),
                new Where('categoria_produto.Tipo', 'Pode vender'),
                new Where('categoria_produto.Site', 'Sim')
            ]);
            $colunasWhere = array_column($consulta->getWhere(), "column"); 
            if(!in_array("produto.Id_Produto_Pai", $colunasWhere)) {
                $where[] = new Where('produto.Id_Produto_Pai', '');
            }
            $consulta = new DatabaseQuery(
                $where,
                $consulta->getOrder(),
                $consulta->getLimit(),
                $consulta->getGroupBy(),
                $consulta->getHaving()
            );
        } else {
            $consulta = new DatabaseQuery(
                [
                    new Where('produto.Ativo', 'Sim'),
                    new Where('produto.Tipo_Produto', 'P'),
                    new Where('produto.Site', 'Sim'),
                    new Where('categoria_produto.Ativo', 'Sim'),
                    new Where('categoria_produto.Tipo_Categoria', 'P'),
                    new Where('categoria_produto.Tipo', 'Pode vender'),
                    new Where('categoria_produto.Site', 'Sim'),
                ],
                [
                    new OrderBy("produto.Data_Cadastro", "DESC")
                ]
            );
        }
        $sql .= $consulta->getFilter();

        $consultarProdutos = BancoDados::consultar(
            $sql,
            $consulta->getFilterValues()
        );
        $obterProdutos = $consultarProdutos->fetchAll(PDO::FETCH_OBJ);
        $produtos = [];
        $havingVariacoes = [];

        if(!Empresa::venderEstoqueNegativo()) {
            $havingVariacoes[] = new Having("Qtd_Estoque", ">", "0");
        }

        foreach($obterProdutos as $itemProduto) {
            $cores = [];
            $tamanhos = [];
            $tipos = [];
            $especificacoes = [];

            $consultaVariacoes = new DatabaseQuery(
                [
                    new Where('Id_Produto_Pai', $itemProduto->Id)
                ]
            );
            $consultaVariacoes->setHaving($havingVariacoes);
            $variacoes = VariacaoProduto::buscarTodas(
                $consultaVariacoes  
            );

            foreach($variacoes as $variacao) {
                if(!empty($variacao->cor) && !in_array($variacao->cor, $cores)) {
                    $cores[] = $variacao->cor;
                }
                if(!empty($variacao->tamanho) &&!in_array($variacao->tamanho, $tamanhos)) {
                    $tamanhos[] = $variacao->tamanho;
                }
                if(!empty($variacao->tipo) &&!in_array($variacao->tipo, $tipos)) {
                    $tipos[] = $variacao->tipo;
                }
                if(!empty($variacao->especificacao) &&!in_array($variacao->especificacao, $especificacoes)) {
                    $especificacoes[] = $variacao->especificacao;
                }
            }

            $codigoProduto = preg_replace("/[^0-9]/", "", $itemProduto->Codigo);
            $imagens = [];

            foreach($colunasFotosProduto as $indice => $fotoProduto) {
                $fotoProduto = str_replace("produto.", "", $fotoProduto);
                
                if(isset($itemProduto->$fotoProduto) && !empty($itemProduto->$fotoProduto)) {
                    $getNumeroFoto = str_replace("produto.Foto", "", $colunasFotosProduto[$indice]);
                    $numeroFoto = strlen($getNumeroFoto) > 0 ? "-$getNumeroFoto" : "";
                    
                    if(file_exists("../../produto/fotos/$codigoProduto{$numeroFoto}/original/{$itemProduto->$fotoProduto}")) {
                        $imagens[] = "https://$itemProduto->Caminho_Foto/produto/fotos/$codigoProduto{$numeroFoto}/original/{$itemProduto->$fotoProduto}";
                    }
                }

            }

            $produto = new Produto(
                $itemProduto->Id ?? '',
                $itemProduto->Titulo ?? '',
                $itemProduto->Preco ?? '',
                $itemProduto->Caracteristica ?? '',
                $itemProduto->Qtd_Estoque ?? 0,
                $cores,
                $tamanhos,
                $tipos,
                $especificacoes,
                $imagens
            );
            $produtos[] = $produto;
        }
        return $produtos;
    }

    public static function buscarPrecos(DatabaseQuery $consulta, array $colunas)
    {
        $colunas = implode(", ", $colunas);
        $tabelaPreco = Empresa::tabelaPreco();
        $colunas = str_replace("Preco", "Preco$tabelaPreco Preco", $colunas);
        $sql =  "SELECT $colunas FROM produto ".$consulta->getFilter();
        
        $consultarProdutos = BancoDados::consultar(
            $sql,
            $consulta->getFilterValues()
        );
        $obterProdutos = $consultarProdutos->fetchAll(PDO::FETCH_OBJ);
        return $obterProdutos;
    }

    public static function quantidadeProdutos(DatabaseQuery $consulta)
    {
        $where = array_merge($consulta->getWhere(), [
            new Where('produto.Ativo', 'Sim'),
            new Where('produto.Tipo_Produto', 'P'),
            new Where('produto.Site', 'Sim'),
            new Where('produto.Id_Produto_Pai', ''),
            new Where('categoria_produto.Ativo', 'Sim'),
            new Where('categoria_produto.Tipo_Categoria', 'P'),
            new Where('categoria_produto.Tipo', 'Pode vender'),
            new Where('categoria_produto.Site', 'Sim')
        ]);
        $consulta = new DatabaseQuery(
            $where,
            $consulta->getOrder(),
            null,
            $consulta->getGroupBy(),
            $consulta->getHaving()
        );
        $sql = "SELECT COUNT(produto.Id) AS quantidade FROM produto
            INNER JOIN categoria_produto ON categoria_produto.Id = produto.Id_Categoria_Produto {$consulta->getFilter()}";
        $consultarProdutos = BancoDados::consultar(
            $sql, 
            $consulta->getFilterValues()
        );
        $obterProdutos = $consultarProdutos->fetchObject();
        return (int)$obterProdutos->quantidade;
    } 
    
    public static function impostos(string $idProduto, int $idRegimeEspecial, int $idUnidade, string $ufUnidade, string $ufCliente)
    {
        $impostos = [];

        $getDadosProduto = BancoDados::consultar("SELECT Id_Ncm, Tabela_Icms, Regime_ST, Codigo_ST, Cest FROM produto WHERE Id = ?", [$idProduto]);
        $dadosProduto = $getDadosProduto->fetchObject();

        if(!empty($getDadosProduto->Id_Ncm)) {
            $getNcm = BancoDados::consultar("SELECT * FROM ncm INNER JOIN ncm_estado ON ncm_estado.Id_Ncm = ncm.Id
                WHERE ncm.Ncm = ? AND ncm.Id_Regime_Especial = ? AND ncm_estado.Id_Unidade = ?  AND ncm_estado.Estado_Dentro = ?
                AND ncm_estado.Estado_Fora = ? AND ncm.Cest = ?",
                [
                    $dadosProduto->Id_Ncm, $idRegimeEspecial, $idUnidade, $ufUnidade, $ufCliente, $dadosProduto->Cest
                ]
            );
            $ncm = $getNcm->fetchObject();

            if(!empty($ncm->CST_V)) {
                $impostos["Icms_Sub"] = $ncm->Icms_Fora_V > 0 ? $ncm->Icms_Dentro_V : $ncm->Icms_Fora_V;
                $impostos["Aliquota_Icms"] = $ncm->Icms_Fora_V > 0 ? $ncm->Icms_Fora_V : $ncm->Icms_Dentro_V;
                $impostos["Mva"] = $ncm->Mva_Fora_V;
                $impostos["Reducao_Icms"] = $ncm->Reducao_Fora_V;
                $impostos["Reducao_BC"] = $ncm->Reducao_Dentro_V;
                $impostos["Codigo_ST"] = $ncm->CST_V;
                $impostos["CST_Ipi"] = $ncm->CST_Ipi_V;
                $impostos["Aliquota_Ipi"] = $ncm->Aliquota_Ipi_V;
                $impostos["CST_Pis"] = $ncm->CST_Pis_V;
                $impostos["Aliquota_Pis"] = $ncm->Aliquota_Pis_V;
                $impostos["CST_Cofins"] = $ncm->CST_Cofins_V;
                $impostos["Aliquota_Cofins"] = $ncm->Aliquota_Cofins_V;
                $impostos["CST_V"] = $ncm->CST_V;
            }
        } else {
            $getIcmsInternoCliente = BancoDados::consultar("SELECT Id, Aliquota_Interna, Aliquota{$dadosProduto->Tabela_Icms} as Aliquota,
                Icms_Sub{$dadosProduto->Tabela_Icms} as Icms_Sub, Reducao{$dadosProduto->Tabela_Icms} as Reducao, ReducaoBC{$dadosProduto->Tabela_Icms} as ReducaoBC FROM icms 
                WHERE Estado = ? AND Id_Unidade = ?", [$ufCliente, $idUnidade]
            );
            $icmsInternoCliente = $getIcmsInternoCliente->fetchObject();
            $impostos["Icms_Sub"] = $icmsInternoCliente->Aliquota_Interna;
            
            if($dadosProduto->Regime_ST == 'Normal' || $dadosProduto->Codigo_ST == '201' || $dadosProduto->Codigo_ST == '202'){
                $impostos["Aliquota_Icms"] = $icmsInternoCliente->Aliquota;
                $impostos["Mva"] = $icmsInternoCliente->Icms_Sub;
                $impostos["Reducao_Icms"] = $icmsInternoCliente->Reducao;
                $impostos["Reducao_BC"] = $icmsInternoCliente->ReducaoBC;
            }
        }
        return (object)$impostos;
    }

    public static function baixarEstoque(string $id, string $idProduto, float $qtd, int $idLocalEstoque, float $precoCusto,
        float $precoVenda, int $idCfop, float $icms, float $ipi, string $origem, string $concluido, string $idExterno,
        int $idUnidade, float $icmsSub, float $mva, float $reducao, string $idCliente
    )
    {
        $getCfop = BancoDados::consultar("SELECT Financeiro FROM cfop WHERE Id = ?", [$idCfop]);
        $gerarFinanceiro = $getCfop->fetchObject()->Financeiro;
        $bonificado = $gerarFinanceiro == "No" ? "Sim" : "No";
        $valorIcms = 0;
        $origemST = 0;

        $getProduto = BancoDados::consultar("SELECT Custo, Titulo, Id_Categoria_Produto, Id_Fornecedor FROM produto WHERE Id = ?", [$idProduto]);
        $dadosProduto = $getProduto->fetchObject();

        if ($precoVenda) {
            $valorIcms = $precoVenda * $qtd * ($icms / 100);
        }

        $idUnidade = empty($idUnidade) ? $_COOKIE["Unidade"] : $idUnidade;
        $dataHoraAtual = date("Y-m-d H:i:s");

        $baixarEstoque = BancoDados::consultar(
            "INSERT INTO estoque SET Id = ?, Id_Produto = ?, Id_Categoria_Produto = ?, Id_Fornecedor = ?, Nome_Produto = ?, Qtd = ?,
                Id_Local_Estoque = ?, Preco_Custo = ?, Preco_Venda = ?, Id_Cfop = ?, Bonificado = ?, Icms = ?, Valor_Icms = ?, Icms_Sub = ?,
                Mva = ?, Reducao = ?, Origem_ST = ?, Ipi = ?, Origem = ?, Concluido = ?, Id_Externo = ?, Id_Unidade = ?,
                Data_Cadastro = ?, Data_Cadastro_Upd = ?, Id_Cliente = ?",
            [
                $id, $idProduto, $dadosProduto->Id_Categoria_Produto, $dadosProduto->Id_Fornecedor, $dadosProduto->Titulo, $qtd, $idLocalEstoque,
                $precoCusto, $precoVenda, $idCfop, $bonificado, $icms, $valorIcms, $icmsSub, $mva, $reducao, $origemST, $ipi, $origem, $concluido,
                $idExterno, $idUnidade, $dataHoraAtual, $dataHoraAtual, $idCliente
            ],
            false
        );

        return $baixarEstoque->rowCount() > 0;
    }

    public static function filtrosOrdenacao()
    {
        $ordens = [
            "produto_mais_recente"=> "Mais Recente",
            "produto_mais_antigo"=> "Mais Antigos",
            "preco_crescente"=> "Preo Mais Baixo",
            "preco_decrescente"=> "Preo Mais Alto",
        ];

        return $ordens;
    }

    public static function destaques()
    {
        $tabelaPreco = Empresa::tabelaPreco();
        $getDestaques = BancoDados::consultar("SELECT produto.Id, produto.Titulo, produto.Foto, produto.Caminho_Foto, produto.Codigo,
            produto.Preco$tabelaPreco Preco FROM produto INNER JOIN categoria_produto ON categoria_produto.Id = produto.Id_Categoria_Produto
            INNER JOIN site_produto_destaque ON site_produto_destaque.Id_Produto = produto.Id
            WHERE produto.Ativo = 'Sim' AND produto.Tipo_Produto = 'P' AND produto.Site = 'Sim' AND categoria_produto.Ativo = 'Sim'
            AND categoria_produto.Tipo_Categoria = 'P' AND categoria_produto.Tipo = 'Pode vender' AND categoria_produto.Site = 'Sim' ORDER BY Id");
        $destaques = $getDestaques->fetchAll(PDO::FETCH_OBJ);
        $produtos = [];

        foreach($destaques as $destaque) {
            $codigoProduto = preg_replace("/[^0-9]/", "", $destaque->Codigo);
            $imagens = [];

            if(!empty($destaque->Foto)) {
                if(file_exists("../../produto/fotos/$codigoProduto/original/{$destaque->Foto}")) {
                    $imagens[] = "https://$destaque->Caminho_Foto/produto/fotos/$codigoProduto/original/{$destaque->Foto}";
                }
            }
            $produtos[] = new Produto(
                $destaque->Id,
                $destaque->Titulo,
                $destaque->Preco,
                "",
                0,
                [],
                [],
                [],
                [],
                $imagens
            );
        }

        return $produtos;
    }

    public static function dimensoes(DatabaseQuery $consulta = null): array
    {
        $sql = "SELECT Id id, Peso_Bruto peso, Altura altura, Largura largura, Comprimento comprimento FROM produto" . $consulta->getFilter();

        $consultarProdutos = BancoDados::consultar(
            $sql,
            $consulta->getFilterValues()
        );
        $produtos = $consultarProdutos->fetchAll(PDO::FETCH_OBJ);

        return $produtos ?? [];
    }
}