<?php
namespace LojaVirtual\App\Model\Type\Customer;

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Database\DatabaseQuery;
use PDO;

class Endereco
{
    public $id;
    public $contato;
    public $telefone;
    public $uf;
    public $cidade;
    public $bairro;
    public $logradouro;
    public $numero;
    public $cep;
    public $complemento;
    public $ibge;
    public $padrao;

    public function __construct(string $contato, string $telefone, string $uf, string $cidade, string $bairro, string $logradouro, string $numero,
        string $cep, string $ibge, ?string $complemento = null, ?string $padrao = null
    )
    {
        $this->contato = $contato;
        $this->telefone = $telefone;
        $this->uf = $uf;
        $this->cidade = $cidade;
        $this->bairro = $bairro;
        $this->logradouro = $logradouro;
        $this->numero = $numero;
        $this->cep = $cep;
        $this->ibge = $ibge;
        $this->complemento = $complemento;
        $this->padrao = $padrao;
    }

    public static function buscar(DatabaseQuery $consulta)
    {
        $sql = "SELECT * FROM endereco_cliente {$consulta->getFilter()}";
        $enderecos = [];

        $consultarEnderecos = BancoDados::consultar($sql, $consulta->getFilterValues());
        $getEnderecos = $consultarEnderecos->fetchAll(PDO::FETCH_OBJ);

        foreach($getEnderecos as $dadosEndereco) {
            $endereco = new Endereco(
                $dadosEndereco->Contato, $dadosEndereco->Telefone, $dadosEndereco->Estado, $dadosEndereco->Cidade,
                $dadosEndereco->Bairro, $dadosEndereco->Rua, $dadosEndereco->Numero, $dadosEndereco->Cep,
                $dadosEndereco->Ibge, $dadosEndereco->Complemento, $dadosEndereco->Padrao
            );
            $endereco->id = $dadosEndereco->Id;
            $enderecos[] = $endereco;
        }
        return $enderecos;
    }

    public function salvar(string $idCliente)
    {
        $qtdEnderecosCliente = BancoDados::consultar("SELECT COUNT(Id) Qtd_Enderecos FROM endereco_cliente WHERE Id_Cliente = ?", [$idCliente]);
        $qtdEnderecosCliente = $qtdEnderecosCliente->fetchObject()->Qtd_Enderecos;
        $padrao = $qtdEnderecosCliente > 0 ? "No" : "Sim";
        $cadastrar = BancoDados::consultar(
            "INSERT INTO endereco_cliente SET Id_Cliente = ?, Contato = ?, Telefone = ?, Cep = ?, Rua = ?, Numero = ?, Complemento = ?,
                Bairro = ?, Cidade = ?, Estado = ?, Ibge = ?, Padrao = ?", 
            [
                $idCliente, $this->contato, $this->telefone, $this->cep, $this->logradouro, $this->numero, $this->complemento, $this->bairro,
                $this->cidade, $this->uf, $this->ibge, $padrao
            ]
        );
        return $cadastrar->rowCount() > 0 ? BancoDados::idInserido() : false;
    }

    public function atualizar(string $idCliente)
    {
        $atualizar = BancoDados::consultar(
            "UPDATE endereco_cliente SET Contato = ?, Telefone = ?, Cep = ?, Rua = ?, Numero = ?, Complemento = ?,
                Bairro = ?, Cidade = ?, Estado = ?, Ibge = ? WHERE Id = ? AND Id_Cliente = ?", 
            [
                $this->contato, $this->telefone, $this->cep, $this->logradouro, $this->numero, $this->complemento, $this->bairro,
                $this->cidade, $this->uf, $this->ibge, $this->id, $idCliente
            ], 
            false
        );
        return $atualizar->rowCount() > 0;
    }

    public static function atualizarPadrao(string $idCliente, int $idEndereco, string $padrao)
    {
        BancoDados::consultar(
            "UPDATE endereco_cliente SET Padrao = ? WHERE Id_Cliente = ? ",
            [
                codificarString("No", "ISO-8859-1", "UTF-8"), $idCliente
            ],
            false
        );
        $atualizar = BancoDados::consultar(
            "UPDATE endereco_cliente SET Padrao = ? WHERE Id = ? AND Id_Cliente = ? ",
            [
                $padrao, $idEndereco, $idCliente
            ],
            false
        );
        return $atualizar->rowCount() > 0;
    }

    public static function excluir(string $idCliente, int $idEndereco)
    {
        $excluirEndereco = BancoDados::consultar(
            "DELETE FROM endereco_cliente WHERE Id = ? AND Id_Cliente = ?",
            [
                $idEndereco, $idCliente
            ]
        );

        return $excluirEndereco->rowCount() > 0;
    }
}