<?php
namespace LojaVirtual\App\Model;

use LojaVirtual\App\Config\Database\BancoDados;
use LojaVirtual\App\Config\Database\DatabaseQuery;
use LojaVirtual\App\Config\Database\Where;
use LojaVirtual\App\Config\Empresa;
use PDO;

class VariacaoProduto
{
    public $id;
    public $idProdutoPai;
    public $cor;
    public $tamanho;
    public $tipo;
    public $especificacao;
    public $quantidadeEstoque;

    public function __construct(string $id, string $idProdutoPai, string $cor, string $tamanho, string $tipo, string $especificacao, float $qtdEstoque)
    {
        $this->id = $id;
        $this->idProdutoPai = $idProdutoPai;
        $this->cor = $cor;
        $this->tamanho = $tamanho;
        $this->tipo = $tipo;
        $this->especificacao = $especificacao;
        $this->quantidadeEstoque = $qtdEstoque;
    }

    public static function buscarTodas(?DatabaseQuery $consulta = null, ?array $colunas = null): array
    {
        if($colunas) {
            $colunas = implode(", ", $colunas);
            $tabelaPreco = Empresa::tabelaPreco();
            $colunas = str_replace("Preco", "Preco$tabelaPreco Preco", $colunas);
        }

        $sqlEstoque = "SELECT SUM(Qtd) FROM estoque WHERE Id_Produto = produto.Id AND Concluido = 'Sim'";
        $sql = $colunas ? "SELECT $colunas FROM produto" : "SELECT produto.Id, produto.Id_Produto_Pai, ($sqlEstoque) Qtd_Estoque,
            produto.Grade_Cor, produto.Grade_Tamanho, produto.Grade_Genero, produto.Genero_Produto, produto.Grade_Especificacao FROM produto";
        $sql .= " INNER JOIN categoria_produto ON produto.Id_Categoria_Produto = categoria_produto.Id";

        if($consulta) {
            $consulta = new DatabaseQuery(
                array_merge($consulta->getWhere(), [
                    new Where('produto.Ativo', 'Sim'),
                    new Where('produto.Tipo_Produto', 'P'),
                    new Where('produto.Site', 'Sim'),
                    new Where('categoria_produto.Ativo', 'Sim'),
                    new Where('categoria_produto.Tipo_Categoria', 'P'),
                    new Where('categoria_produto.Tipo', 'Pode vender'),
                    new Where('categoria_produto.Site', 'Sim')
                ]),
                $consulta->getOrder(),
                $consulta->getLimit(),
                $consulta->getGroupBy(),
                $consulta->getHaving()
            );
        } else {
            $consulta = new DatabaseQuery(
                [
                    new Where('produto.Ativo', 'Sim'),
                    new Where('produto.Tipo_Produto', 'P'),
                    new Where('produto.Site', 'Sim'),
                    new Where('categoria_produto.Ativo', 'Sim'),
                    new Where('categoria_produto.Tipo_Categoria', 'P'),
                    new Where('categoria_produto.Tipo', 'Pode vender'),
                    new Where('categoria_produto.Site', 'Sim')
                ]
            );
        }
        $sql .= $consulta->getFilter();

        $consultarVariacoes = BancoDados::consultar(
            $sql,
            $consulta->getFilterValues()
        );
        $obterVariacoes = $consultarVariacoes->fetchAll(PDO::FETCH_OBJ);
        $variacoes = [];

        foreach($obterVariacoes as $itemVariacao) {
            $variacao = new VariacaoProduto(
                $itemVariacao->Id,
                $itemVariacao->Id_Produto_Pai,
                $itemVariacao->Grade_Cor,
                $itemVariacao->Grade_Tamanho,
                $itemVariacao->Grade_Genero,
                $itemVariacao->Grade_Especificacao,
                (float)$itemVariacao->Qtd_Estoque
            );
            $variacoes[] = $variacao;
        }
        return $variacoes;
    }

    public static function buscarPrecos(array $filtros, array $colunas)
    {
        $colunas = implode(", ", $colunas);
        $tabelaPreco = Empresa::tabelaPreco();
        $colunas = str_replace("Preco", "Preco$tabelaPreco Preco", $colunas);
        $colunas = str_replace("Titulo", "(SELECT Titulo FROM produto produto_pai WHERE produto_pai.Id = produto.Id_Produto_Pai) Titulo", $colunas);
        $sql =  "SELECT $colunas FROM produto WHERE";

        foreach($filtros as $indice => $filtro) {
            $sql .= ($indice > 0 ? " OR " : "");

            foreach($filtro as $chave => $where) {
                $sql .= ($chave == 0 ? " (" : "") . ($chave > 0 ? " AND " : '') . "$where->column = '$where->value'";
                $sql .= $chave + 1 == sizeof($filtro) ? ")" : "";
            }
        }
        
        $consultarVariacoes = BancoDados::consultar(
            $sql
        );
        $obterVariacoes = $consultarVariacoes->fetchAll(PDO::FETCH_OBJ);
        return $obterVariacoes;
    }
}