<?php
namespace MercadoPago\Src;

use MercadoPago\Src\Config\Autenticacao;
use MercadoPago\Src\Config\Http\Api;
use MercadoPago\Src\Config\Http\Request;
use MercadoPago\Src\Entidade\Pagador;


/**
 * @var float $transaction_amount
 * @var ?string $token
 * @var ?string $description
 * @var ?int $installments
 * @var string $payment_method_id
 * @var ?string $issuer_id
 * @var Pagador $payer
 */
class Pagamento {
    public $transaction_amount;
    public $token;
    public $description;
    public $installments;
    public $payment_method_id;
    public $issuer_id;
    public $payer;
    public $additional_info;
    public $notification_url;
    public $external_reference;

    public function __construct(float $transaction_amount, Pagador $payer, string $payment_method_id, string $idExterno, ?string $token = null,
        ?int $installments = null, ?string $description = null, ?string $issuer_id = null, ?object $additional_info = null, ?string $urlNotificacao = null
    )
    {
        $this->transaction_amount = $transaction_amount;
        $this->payer = $payer;
        $this->payment_method_id = $payment_method_id;
        $this->external_reference = $idExterno;
        $this->token = $token;
        $this->installments = $installments;
        $this->description = $description;
        $this->issuer_id = $issuer_id;
        $this->additional_info = $additional_info;
        $this->notification_url = $urlNotificacao;
    }

    public function processar(): object
    {
        $propriedadesObjeto = get_object_vars($this);
        $url = Api::getUrl()."/payments";
        $idPagamento = md5(microtime().date('Y-m-d H:i:s'));
        $tokenAcesso = Autenticacao::getTokenAcesso();
        $cabecalho = [
            "Authorization: Bearer $tokenAcesso",
            "Content-Type: application/json",
            "X-Idempotency-Key: $idPagamento"
        ];
        $dados = array_filter($propriedadesObjeto,function($propriedade) {
            if($propriedade !== null) {
                return $propriedade;
            }
        });

        $processarPagamento = Request::post($url, $cabecalho, json_encode($dados));
        
        return $processarPagamento->body;
    }

    public static function getTokenAcesso()
    {
        return Autenticacao::getTokenAcesso();
    }
}