<?php
namespace Sicacred\App\Config\Http;

class Request {
    private static $curl;

    function __construct(){}

    /**
     * Requisio post
     *
     * @param string $url Endpoint de destino
     * @param string $token Token de autenticao
     * @param string $dados Corpo da requisio
     * @param string $tipoConteudo Tipo do conteudo (x-www-form-urlencoded ou json)
     * @return object Cdigo http e body da resposta
     */
    public static function post($url, $token, $dados, $tipoConteudo = ""): object {
        $tiposConteudo = ["x-www-form-urlencoded", "json"];
        $cabecalhoSolicitacao = [
            "Authorization: Bearer $token",
        ];
        if(in_array($tipoConteudo, $tiposConteudo)) {
            array_push($cabecalhoSolicitacao, "Content-Type: application/$tipoConteudo");
        }
        self::$curl = curl_init();
        curl_setopt_array(self::$curl, [
            CURLOPT_URL => "$url",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_HTTPHEADER => $cabecalhoSolicitacao,
            CURLOPT_POSTFIELDS => $dados,
        ]);
        
        $response = curl_exec(self::$curl);
        $httpcode = curl_getinfo(self::$curl)['http_code'];
        curl_close(self::$curl);
        return (object)["http_code"=> $httpcode, "body"=> json_decode($response)];
    }

    /**
     * Requisio get
     *
     * @param string $url Endpoint de destino
     * @param string $token Token de autenticao
     * @return object Cdigo http e body da resposta
     */
    public static function get($url, $token = null): object {
        self::$curl = curl_init();
        $configResquest = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
        ];
        if($token) {
            $configResquest[CURLOPT_HTTPHEADER] = [
              "Authorization: Bearer $token"
            ];
        }
        curl_setopt_array(self::$curl, $configResquest);
          
        $response = curl_exec(self::$curl);
        $httpcode = curl_getinfo(self::$curl)['http_code'];
        curl_close(self::$curl);

        return (object)["http_code"=> $httpcode, "body"=> json_decode($response)];
    }
}